/******************************************************************************************************

 Cerebrum.Vocabulary.Desktop.dll is part of the Morphology .NET toolkit and contains implementation of
 the toolkit user interface for the OOKBMS Cerebrum desktop environment.

 Copyright (C) 2007-2009 Shuklin Dmitry Evgenievich and Boyko Sergey Vasilievich. All rights reserved.

 The Morphology .NET toolkit is free software; you can redistribute it and/or modify it under the terms 
 of the Microsoft Public License (Ms-PL) as published by the Microsoft.

 This toolkit is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
 QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE 
 COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

 See the Microsoft Public License (Ms-PL) for more details.

 You should have received a copy of the Microsoft Public License (Ms-PL) along with this toolkit;
 if not, the Microsoft Public License (Ms-PL) can be found at 
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL

 Please see the LICENSE.txt file for the full license.

 For all inquieries and concerns contact an authors at  cerebrum@shdsoft.ru

 This copyright notice MUST APPEAR in all copies of this file!

 //////////////////////////////////////////////////////////////////////////////////////////////////////

  Cerebrum.Vocabulary.Desktop.dll -   Morphology .NET toolkit  
          Cerebrum.

 Copyright (C) 2007-2009       .   .

      ;     / 
      Microsoft Public License (Ms-PL).

     ,    ,   - ;
           .  
          .    
 ,        ,   
 .

      Microsoft Public License (Ms-PL)

      Microsoft Public License (Ms-PL)    ; 
   ,  Microsoft Public License (Ms-PL)      
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

       LICENSE.txt

          : cerebrum@shdsoft.ru
 
          !
 
******************************************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Cerebrum.WorkWithFormsOfDatabases;

namespace Cerebrum.Vocabulary
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class AppUiService : Cerebrum.Windows.Forms.Reflection.UiServiceDescriptor 
	{
		static AppUiService()
		{
			Cerebrum.Runtime.Semiotics.KernelObjectKindId.Register();
		}

		public AppUiService()
		{
		}

		public override void ComponentActivated(object component)
		{
			Cerebrum.Windows.Forms.IPrimaryWindow primaryWindow = (Cerebrum.Windows.Forms.IPrimaryWindow)Cerebrum.Windows.Forms.Application.Instance.PrimaryWindow;
			Cerebrum.Windows.Forms.IUiControl cmdImport = primaryWindow.GetCommand("Import");
			Cerebrum.Windows.Forms.IUiControl cmdExport = primaryWindow.GetCommand("Export");
			Cerebrum.Windows.Forms.IUiControl cmdTest = primaryWindow.GetCommand("Test_Message");
			Cerebrum.Windows.Forms.IUiControl cmdCreateWL = primaryWindow.GetCommand("CreateWordsList");

			//    ...
			if (component is Cerebrum.Vocabulary.VocabularyDatabaseForm)
			{
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdImport, true);
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdExport, true);
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdTest, true);
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdCreateWL, true);
			}
			else if (component == null)
			{
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdImport, false);
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdExport, false);
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdTest, false);
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdCreateWL, false);
			}
			else
			{
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdImport, false);
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdExport, false);
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdTest, false);
				Cerebrum.Windows.Forms.Utilites.SetControlEnabledProperty(cmdCreateWL, false);
			}

			base.ComponentActivated (component);
		}


		public override bool CommandActivated(Cerebrum.IConnector message, object sender, object argument)
		{
			string name = ((Cerebrum.Reflection.MessageDescriptor)message.Component).Name;

			switch(name)
			{
				case "New":
				{
					//Cerebrum.Integrator.DomainContext dc = CreateDocument("Cerebrum.Vocabulary.Database.Master");
					Cerebrum.Integrator.DomainContext dc = null;
					string DomainName = "Cerebrum.Vocabulary.Database.Master";
					string TemplateFile = System.IO.Path.Combine(System.IO.Path.GetDirectoryName(typeof(AppUiService).Assembly.Location), @"Cerebrum.Vocabulary.Master.xdb");
					Cerebrum.Windows.Forms.Application app = Cerebrum.Windows.Forms.Application.Instance;
					dc = Cerebrum.WorkWithFormsOfDatabases.CreateAndOpenDB.CreateDocument(app, DomainName, TemplateFile);
					if(dc != null)
					{
						Open(dc).PreInit();
					}
					return true;
				}
				case "Open":
				{
					//Cerebrum.Integrator.DomainContext dc = OpenDocument(argument);
					Cerebrum.Integrator.DomainContext dc = null;
					dc = Cerebrum.WorkWithFormsOfDatabases.CreateAndOpenDB.OpenDocument(argument);
					if(dc != null)
					{
						Open(dc).PreOpen();
					}
					return true;
				}
				case "Import":
				{
					//  .
					Cerebrum.Vocabulary.VocabularyDatabaseForm dbForm = PickupVocDbForm();
					if(dbForm != null)
					{
						Cerebrum.Vocabulary.Desktop.ImportCSVForm importForm = new Cerebrum.Vocabulary.Desktop.ImportCSVForm(dbForm);
						importForm.ShowDialog();
					}
					return true;
				}
				case "Export":
				{
					Cerebrum.Vocabulary.VocabularyDatabaseForm dbForm = PickupVocDbForm();
					if(dbForm!=null) dbForm.ExportCSVFiles();
					return true;
				}
				case "Test_Message":
				{
					Cerebrum.Vocabulary.VocabularyDatabaseForm dbForm = PickupVocDbForm();
					if(dbForm!=null) dbForm.Test();
					return true;
				}
				case "CreateWordsList":
				{
					Cerebrum.Vocabulary.VocabularyDatabaseForm dbForm = PickupVocDbForm();
					if(dbForm!=null) dbForm.CreateWordsList();
					return true;
				}
			}
			return base.CommandActivated(message, sender, argument);
		}

		private static Cerebrum.Vocabulary.VocabularyDatabaseForm Open(Cerebrum.Integrator.DomainContext c)
		{
			Cerebrum.Vocabulary.VocabularyDatabaseForm dlg1 = new Cerebrum.Vocabulary.VocabularyDatabaseForm();
			((Cerebrum.IComponent)dlg1).SetConnector(Cerebrum.SerializeDirection.Init, new Cerebrum.Runtime.SurrogateComposite(c.Workspace));
			dlg1.MdiParent = Cerebrum.Windows.Forms.Application.Instance.PrimaryWindow;
			dlg1.Show();

			return dlg1;
		}

		private Cerebrum.Vocabulary.VocabularyDatabaseForm PickupVocDbForm()
		{
			Cerebrum.Windows.Forms.Application app = this.DomainContext.Application as Cerebrum.Windows.Forms.Application;
			if(app != null)
			{
				Cerebrum.DesktopEngine.PrimaryWindowForm form = app.PrimaryWindow as Cerebrum.DesktopEngine.PrimaryWindowForm;
				if(form != null)
				{
					return form.ActiveComponent as Cerebrum.Vocabulary.VocabularyDatabaseForm;
				}
			}
			return null;
		}

		/// <summary>
		///        .
		/// </summary>
		/// <param name="File"></param>
		/// <returns></returns>
		public Cerebrum.Vocabulary.Desktop.FileInfo LoadFile(string File)
		{
			System.IO.FileStream fs;
			System.IO.StreamReader sr;
			string Text = "";
			Cerebrum.Vocabulary.Desktop.FileInfo fi = null;

			try
			{
				//  .
				fs = new System.IO.FileStream(File, System.IO.FileMode.Open);
				sr = new System.IO.StreamReader(fs, System.Text.Encoding.GetEncoding("windows-1251"));
				Text = sr.ReadToEnd();
				sr.Close();
				fs.Close();

				//    .
				int n = File.LastIndexOf(@"\");
				fi = new Cerebrum.Vocabulary.Desktop.FileInfo();
				fi.FileName = File.Substring(n + 1);
				fi.FileText = Text;
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
			return fi;
		}

		/// <summary>
		///        .
		/// </summary>
		/// <param name="DefaultExt"></param>
		/// <param name="Filter"></param>
		/// <returns></returns>
		public Cerebrum.Vocabulary.Desktop.FileInfo LoadFile(string DefaultExt, string Filter)
		{
			OpenFileDialog dlg;
			Cerebrum.Vocabulary.Desktop.FileInfo fi = null;

			try
			{
				dlg = new OpenFileDialog();
				dlg.DefaultExt = DefaultExt;
				dlg.Filter = Filter;
				if (dlg.ShowDialog() == DialogResult.OK)
				{
					fi = LoadFile(dlg.FileName);
				}
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
			return fi;
		}
	}
}
