/******************************************************************************************************

 Cerebrum.Vocabulary.Desktop.dll is part of the Morphology .NET toolkit and contains implementation of
 the toolkit user interface for the OOKBMS Cerebrum desktop environment.

 Copyright (C) 2007-2009 Shuklin Dmitry Evgenievich and Boyko Sergey Vasilievich. All rights reserved.

 The Morphology .NET toolkit is free software; you can redistribute it and/or modify it under the terms 
 of the Microsoft Public License (Ms-PL) as published by the Microsoft.

 This toolkit is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
 QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE 
 COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

 See the Microsoft Public License (Ms-PL) for more details.

 You should have received a copy of the Microsoft Public License (Ms-PL) along with this toolkit;
 if not, the Microsoft Public License (Ms-PL) can be found at 
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL

 Please see the LICENSE.txt file for the full license.

 For all inquieries and concerns contact an authors at  cerebrum@shdsoft.ru

 This copyright notice MUST APPEAR in all copies of this file!

 //////////////////////////////////////////////////////////////////////////////////////////////////////

  Cerebrum.Vocabulary.Desktop.dll -   Morphology .NET toolkit  
          Cerebrum.

 Copyright (C) 2007-2009       .   .

      ;     / 
      Microsoft Public License (Ms-PL).

     ,    ,   - ;
           .  
          .    
 ,        ,   
 .

      Microsoft Public License (Ms-PL)

      Microsoft Public License (Ms-PL)    ; 
   ,  Microsoft Public License (Ms-PL)      
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

       LICENSE.txt

          : cerebrum@shdsoft.ru
 
          !
 
******************************************************************************************************/

using System;

namespace Cerebrum.Vocabulary.Desktop
{
	/// <summary>
	/// Summary description for ExportCSV.
	/// </summary>
	public class ExportCSV
	{
		private Cerebrum.Integrator.DomainContext m_DomainContext;
		private Cerebrum.Runtime.HFacade m_WMaster;

		public ExportCSV(Cerebrum.Integrator.DomainContext DomainContext, Cerebrum.Runtime.HFacade hfacade)
		{
			m_DomainContext = DomainContext;
			m_WMaster = hfacade;
		}

		public void ExportZlAttributes(string AppPath)
		{
			System.Collections.SortedList sl = new System.Collections.SortedList();

			try
			{
				using(Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlAttributes"))
				{
					for(int r=0; r<view.Count; r++)
					{
						Cerebrum.Data.ComponentItemView item = view[r];
						string name = item["Name"].ToString();
						string row = name + ";";
						row += item["DisplayName"].ToString() + ";";
						row += item["Description"].ToString();
						sl.Add(name + " " + r.ToString(), row);
					}
				}

				using(System.IO.FileStream fs = new System.IO.FileStream(AppPath + @"\Export\ZlAttributes.csv", System.IO.FileMode.Create))
				{
					using(System.IO.StreamWriter sw = new System.IO.StreamWriter(fs, System.Text.Encoding.GetEncoding("windows-1251")))
					{

						sw.WriteLine("Name;DisplayName;Description");
						for(int i=0; i<sl.Count; i++)
						{
							string row = Convert.ToString(sl.GetByIndex(i));
							sw.WriteLine(row);
						}
			
						sw.Flush();
						fs.Flush();

						sw.Close();
					}
					fs.Close();
				}
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
		}

		public void ExportZlSpecifiers(string AppPath)
		{
			System.Collections.SortedList sl = new System.Collections.SortedList();

			try
			{
				using(Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlSpecifiers"))
				{
					for(int r=0; r<view.Count; r++)
					{
						string name = view[r]["Name"].ToString();
						string Description = view[r]["Description"].ToString();
						Cerebrum.Data.VectorView attributes = (Cerebrum.Data.VectorView)view[r]["ZlAttributes"];
						string attrsName = "";
						string attrsDisplayName = "";
						for(int ar=0; ar<attributes.Count; ar++)
						{
							attrsName += attributes[ar]["Name"].ToString() + ", ";
							attrsDisplayName += attributes[ar]["DisplayName"].ToString() + ", ";
						}
						attrsName = attrsName.Substring(0, attrsName.Length - 2);
						attrsDisplayName = attrsDisplayName.Substring(0, attrsDisplayName.Length - 2);
						if(Description != "")
						{
							Description = attrsDisplayName + "|" + Description;
						}
						else
						{
							Description = attrsDisplayName;
						}

						string row = name + ";" + attrsName + ";" + Description;

						sl.Add(name + " " + r.ToString(), row);
					}
				}

				using(System.IO.FileStream fs = new System.IO.FileStream(AppPath + @"\Export\ZlSpecifiers.csv", System.IO.FileMode.Create))
				{
					using(System.IO.StreamWriter sw = new System.IO.StreamWriter(fs, System.Text.Encoding.GetEncoding("windows-1251")))
					{
						sw.WriteLine("Name;Attributes;Description");
						for(int i=0; i<sl.Count; i++)
						{
							string row = Convert.ToString(sl.GetByIndex(i));
							sw.WriteLine(row);
						}

						sw.Flush();
						fs.Flush();

						sw.Close();
					}
					fs.Close();
				}
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
		}

		public void ExportZlTransforms(string AppPath)
		{
			try
			{
				Cerebrum.ObjectHandle PrefixTransTypeHandle = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Types", "Name", "ZlPrefixTransform");
				Cerebrum.ObjectHandle EndingTransTypeHandle = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Types", "Name", "ZlEndingTransform");
				Cerebrum.ObjectHandle wordBaseAttrHandle = Cerebrum.Management.Utilites.ResolveHandle(this.m_DomainContext, "Attributes", "Name", "WordBase");
				Cerebrum.ObjectHandle primSpecAttrHandle = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Attributes", "Name", "PrimarySpecifier");

				System.Collections.Hashtable htAttrListInZlTransforms = new System.Collections.Hashtable();

				System.IO.FileStream fs = new System.IO.FileStream(AppPath + @"\Export\ZlTransforms.csv", System.IO.FileMode.Create);
				System.IO.StreamWriter sw = new System.IO.StreamWriter(fs, System.Text.Encoding.GetEncoding("windows-1251"));

				sw.WriteLine("Name;Type;Specifier;Wordform;Flexia");

				//        ZlTransforms.
				using(Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlSpecifiers"))
				{
					for(int r=0; r<view.Count; r++)
					{
						Cerebrum.Data.ComponentItemView item = view[r];
						htAttrListInZlTransforms.Add(item.ObjectHandle, Convert.ToString(item["Name"]));
					}
				}

				System.Collections.Hashtable htTransPrimarySpec = new System.Collections.Hashtable();
				System.Collections.SortedList slTransforms = new System.Collections.SortedList();
				//    ZlTransforms.
				using(Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlTransforms"))
				{
					for(int r=0; r<view.Count; r++)
					{
						Cerebrum.Data.ComponentItemView item = view[r];
						//       .
						System.Collections.SortedList slSpecList = new System.Collections.SortedList();

						string TransformName = string.Empty;
						string wordBase = string.Empty;
						FlexiaMode flexiaMode = FlexiaMode.None;
						using(Cerebrum.IContainer cont = (Cerebrum.IContainer)item.GetComposite())
						{
							TransformName = Convert.ToString(Cerebrum.Integrator.Utilites.ObtainComponent(cont, Cerebrum.Specialized.Concepts.NameAttribute));

							try
							{
								Cerebrum.ObjectHandle htmp = ((Cerebrum.IComposite)cont).TypeIdHandle;
								if(htmp == EndingTransTypeHandle)
								{
									//      ...
									flexiaMode = FlexiaMode.Ending;
								}
								else if(htmp == PrefixTransTypeHandle)
								{
									flexiaMode = FlexiaMode.Prefix;
								}
								using(Cerebrum.IComposite compW = cont.AttachConnector(wordBaseAttrHandle))
								{
									if(compW!=null)
									{
										if(this.m_WMaster!=null)
										{
											wordBase = Utilites.FilterNodeText(this.m_WMaster.ObtainText(compW));
										}
										else
										{
											wordBase = Convert.ToString(compW.Component);
										}
									}
								}

								// PrimarySpecifier. Ordinal != 0.
								string PrimarySpecifierName = htAttrListInZlTransforms[Cerebrum.Integrator.Utilites.ObtainComponent(cont, primSpecAttrHandle)].ToString();

								// Ordinal > 0.
								foreach(System.Collections.DictionaryEntry de in (System.Collections.IEnumerable)cont)
								{
									Cerebrum.ObjectHandle hspecifier = (Cerebrum.ObjectHandle)de.Key;
									string SpecCode = Convert.ToString(htAttrListInZlTransforms[hspecifier]);
									if(SpecCode!=null && SpecCode!=string.Empty)
									{
										string flexia = null;
										using(Cerebrum.IComposite compW = cont.AttachConnector(hspecifier))
										{
											if(compW!=null)
											{
												if(this.m_WMaster!=null)
												{
													flexia = Utilites.FilterNodeText(this.m_WMaster.ObtainText(compW));
												}
												else
												{
													flexia = Convert.ToString(compW.Component);
												}
											}
										}
										if(flexia!=null)
										{
											//           .
											SpecifierInfoForTransform sift = new SpecifierInfoForTransform();
											sift.TransformName = TransformName;
											sift.Ordinal = 1;
											sift.SpecifierName = SpecCode;
											sift.WordBase = wordBase;
											sift.Flexia = Convert.ToString(flexia);
											sift.FlexiaMode = flexiaMode;

											if(SpecCode == PrimarySpecifierName)
											{
												//         .
												htTransPrimarySpec.Add(TransformName, sift);
											}
											else
											{
												//         .
												slSpecList.Add(SpecCode, sift);
											}
										}
									}
								}
								slTransforms.Add(TransformName, slSpecList);
							}
							catch(System.Exception ex)
							{
								throw new System.Exception("Export of transform '" + TransformName + "' failed.", ex);
							}
						}
					}
				}

				for(int i=0; i<slTransforms.Count; i++)
				{
					//   .
					string TransformName = slTransforms.GetKey(i).ToString();
					//       .
					SpecifierInfoForTransform sift = (SpecifierInfoForTransform)htTransPrimarySpec[TransformName];
					//  ,    . 
					string Type = "0"; 
					//      .
					//    .
					//    ...
					if(sift.FlexiaMode == FlexiaMode.Ending)
					{
						//   1 (). 
						Type = Cerebrum.Vocabulary.Concepts.KnownValues.EndingMode;
						sw.WriteLine(sift.TransformName + ";" + Type + ";" + sift.SpecifierName + ";" + sift.WordBase + sift.Flexia + ";" + sift.Flexia);
					}
					else if(sift.FlexiaMode == FlexiaMode.Prefix)
					{
						//    2 (). 
						Type = Cerebrum.Vocabulary.Concepts.KnownValues.PrefixMode;
						sw.WriteLine(sift.TransformName + ";" + Type + ";" + sift.SpecifierName + ";" + sift.Flexia + sift.WordBase + ";" + sift.Flexia);
					}
					else
					{
						// 0  Type        ,     
						sw.WriteLine(sift.TransformName + ";" + Type + ";" + sift.SpecifierName + ";" + ";" + sift.Flexia);
					}

					//         .
					System.Collections.SortedList slSpecList = (System.Collections.SortedList)slTransforms.GetByIndex(i);
					for(int i2=0; i2<slSpecList.Count; i2++)
					{
						//       .
						sift = (SpecifierInfoForTransform)slSpecList.GetByIndex(i2);
						//    .
						if(sift.FlexiaMode==FlexiaMode.Ending)
						{
							sw.WriteLine(sift.TransformName + ";" + "0" + ";" + sift.SpecifierName + ";" + sift.WordBase + sift.Flexia + ";" + sift.Flexia);
						}
						else if(sift.FlexiaMode==FlexiaMode.Prefix)
						{
							sw.WriteLine(sift.TransformName + ";" + "0" + ";" + sift.SpecifierName + ";" + sift.Flexia + sift.WordBase + ";" + sift.Flexia);
						}
						else
						{
							sw.WriteLine(sift.TransformName + ";" + "0" + ";" + sift.SpecifierName + ";" + ";" + sift.Flexia);
						}
					}
				}
				sw.Flush();
				sw.Close();
				fs.Close();
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
		}

		public void ExportZlStatements(string AppPath, bool IsAddFlexias)
		{
			System.Collections.Hashtable htStatements = new System.Collections.Hashtable();
			System.Collections.SortedList slStatements = new System.Collections.SortedList();

			try
			{
				Cerebrum.ObjectHandle PrefixTransAttrHandle = Cerebrum.Management.Utilites.ResolveHandle(this.m_DomainContext, "Attributes", "Name", Cerebrum.Vocabulary.Concepts.KnownNames.PrefixModel);
				Cerebrum.ObjectHandle EndingTransAttrHandle = Cerebrum.Management.Utilites.ResolveHandle(this.m_DomainContext, "Attributes", "Name", Cerebrum.Vocabulary.Concepts.KnownNames.EndingModel);
				Cerebrum.ObjectHandle specifierAttrHandle = Cerebrum.Management.Utilites.ResolveHandle(this.m_DomainContext, "Attributes", "Name", "PrimarySpecifier");
				Cerebrum.ObjectHandle wordBaseAttrHandle = Cerebrum.Management.Utilites.ResolveHandle(this.m_DomainContext, "Attributes", "Name", "WordBase");
				using(Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlStatements"))
				{
					for(int r=0; r<view.Count; r++)
					{
						Cerebrum.Data.ComponentItemView item = view[r];
						using(Cerebrum.IComposite comp = item.GetComposite())
						{
							Cerebrum.ObjectHandle SpecHandle = Cerebrum.ObjectHandle.Null;
							string prefix = string.Empty;
							string ending = string.Empty;
							string wordBase = string.Empty;
							string PrefixTransName = string.Empty;
							string EndingTransName = string.Empty;
							using(Cerebrum.IComposite compPT = ((Cerebrum.IContainer)comp).AttachConnector(PrefixTransAttrHandle))
							{
								if(compPT != null)
								{
									Cerebrum.Vocabulary.ZlTransform ptr = (Cerebrum.Vocabulary.ZlTransform)compPT.Component;
									PrefixTransName = ptr.GetAttributeVal("Name").ToString();
									SpecHandle = (Cerebrum.ObjectHandle)Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)compPT, specifierAttrHandle);
									prefix = Convert.ToString(Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)compPT, SpecHandle));
								}
							}
							using(Cerebrum.IComposite compET = ((Cerebrum.IContainer)comp).AttachConnector(EndingTransAttrHandle))
							{
								if(compET != null)
								{
									Cerebrum.Vocabulary.ZlTransform etr = (Cerebrum.Vocabulary.ZlTransform)compET.Component;
									EndingTransName = etr.GetAttributeVal("Name").ToString();
									SpecHandle = (Cerebrum.ObjectHandle)Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)compET, specifierAttrHandle);
									ending = Convert.ToString(Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)compET, SpecHandle));
								}
							}
							using(Cerebrum.IComposite compW = ((Cerebrum.IContainer)comp).AttachConnector(wordBaseAttrHandle))
							{
								if(this.m_WMaster!=null)
								{
									wordBase = Utilites.FilterNodeText(this.m_WMaster.ObtainText(compW));
								}
								else
								{
									wordBase = Convert.ToString(compW.Component);
								}
							}
							System.Collections.ArrayList al = null;
							string SLKey = PrefixTransName + ";" + EndingTransName;
							if(slStatements.ContainsKey(SLKey) == false)
							{
								al = new System.Collections.ArrayList();
								slStatements.Add(SLKey, al);
							}
							else
							{
								al = (System.Collections.ArrayList)slStatements[SLKey];
							}
							if(al != null)
							{
								if(IsAddFlexias == true)
								{
									al.Add(prefix + wordBase + ending);
								}
								else
								{
									al.Add(wordBase);
								}
							}
						}
					}
				}

				System.IO.FileStream fs = new System.IO.FileStream(AppPath + @"\Export\ZlStatements.ini", System.IO.FileMode.Create);
				System.IO.StreamWriter sw = new System.IO.StreamWriter(fs, System.Text.Encoding.GetEncoding("windows-1251"));

				foreach(System.Collections.DictionaryEntry de in slStatements)
				{
					sw.WriteLine("[" + de.Key.ToString() + "]");
					System.Collections.ArrayList al = (System.Collections.ArrayList)de.Value;
					al.Sort();
					for(int i=0; i<al.Count; i++)
					{
						sw.WriteLine(al[i].ToString());
					}
				}

				sw.Flush();
				sw.Close();
				fs.Close();
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
		}
	}
}
