/******************************************************************************************************

 Cerebrum.Vocabulary.Desktop.dll is part of the Morphology .NET toolkit and contains implementation of
 the toolkit user interface for the OOKBMS Cerebrum desktop environment.

 Copyright (C) 2007-2009 Shuklin Dmitry Evgenievich and Boyko Sergey Vasilievich. All rights reserved.

 The Morphology .NET toolkit is free software; you can redistribute it and/or modify it under the terms 
 of the Microsoft Public License (Ms-PL) as published by the Microsoft.

 This toolkit is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
 QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE 
 COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

 See the Microsoft Public License (Ms-PL) for more details.

 You should have received a copy of the Microsoft Public License (Ms-PL) along with this toolkit;
 if not, the Microsoft Public License (Ms-PL) can be found at 
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL

 Please see the LICENSE.txt file for the full license.

 For all inquieries and concerns contact an authors at  cerebrum@shdsoft.ru

 This copyright notice MUST APPEAR in all copies of this file!

 //////////////////////////////////////////////////////////////////////////////////////////////////////

  Cerebrum.Vocabulary.Desktop.dll -   Morphology .NET toolkit  
          Cerebrum.

 Copyright (C) 2007-2009       .   .

      ;     / 
      Microsoft Public License (Ms-PL).

     ,    ,   - ;
           .  
          .    
 ,        ,   
 .

      Microsoft Public License (Ms-PL)

      Microsoft Public License (Ms-PL)    ; 
   ,  Microsoft Public License (Ms-PL)      
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

       LICENSE.txt

          : cerebrum@shdsoft.ru
 
          !
 
******************************************************************************************************/

using System;
using System.Windows.Forms;

namespace Cerebrum.Vocabulary.Desktop
{
	/// <summary>
	/// Summary description for ImportCSV.
	/// </summary>
	public class ImportCSV
	{
		private Cerebrum.Integrator.DomainContext m_DomainContext;
		private Cerebrum.Runtime.ISemiotics m_WMaster;
		private System.Collections.Hashtable htTransformMap;

		public ImportCSV(Cerebrum.Integrator.DomainContext DomainContext, Cerebrum.Runtime.ISemiotics WMaster)
		{
			m_DomainContext = DomainContext;
			m_WMaster = WMaster;
			//     .
			htTransformMap = new System.Collections.Hashtable();
		}

		public void ImportData(string TableName, FileInfo fi, string DupleTableName)
		{
			//string TableName;
			Cerebrum.Data.TableView view;
			Cerebrum.ObjectHandle[] ohAttributes;
			string[] Rows;
			string[] FieldsNameInCSV;
			char[] chEnter = new char[]{Convert.ToChar(13), Convert.ToChar(10)};

			try
			{
				//   .
				int n = fi.FileName.LastIndexOf(".");
				//    .
				Rows = fi.FileText.Split(chEnter);
				//      *.csv.
				FieldsNameInCSV = Rows[0].Split(new char[]{';'});
				//  .
				ohAttributes = new Cerebrum.ObjectHandle[FieldsNameInCSV.Length];
				for(int c=0; c<FieldsNameInCSV.Length; c++)
				{
					if(fi.FieldsInfo.Contains(FieldsNameInCSV[c].Trim(), FindFieldContains.FieldNameInCSV) == false)
					{
						ohAttributes[c] = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Attributes", "Name", FieldsNameInCSV[c].Trim());
					}
					else
					{
						string AttrName = fi.FieldsInfo.GetAttributeName(FieldsNameInCSV[c].Trim());
						ohAttributes[c] = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Attributes", "Name", AttrName);
					}
				}
				//  .
				using(view = Cerebrum.Management.Utilites.GetView(m_DomainContext, TableName))
				{
					Cerebrum.ObjectHandle PrimarySpecifier = Cerebrum.ObjectHandle.Null;

					for(int r=1; r<Rows.Length; r++)
					{
						if(Rows[r] != "")
						{
							string[] AttrVals = Rows[r].Split(new char[]{';'});

							Cerebrum.Data.ComponentItemView item = view.AddNew();
							//    .
							PrimarySpecifier = FillAttributes(fi, item, TableName, FieldsNameInCSV, ohAttributes, AttrVals);
							//     ...
							if(DupleTableName != "")
							{
								//        DupleTableName ,    .
								using (Cerebrum.Data.TableView view1 = Cerebrum.Management.Utilites.GetView(m_DomainContext, DupleTableName))
								{
									view1.EngageConnector(item.ObjectHandle, item.GetComposite());
									Cerebrum.Management.Utilites.RefreshHandle(m_DomainContext, DupleTableName, "Name", item.ObjectHandle);
								}
							}
						}
					}
				}
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
		}

		public void ImportZlSpecifiers(FileInfo fi)
		{
			Cerebrum.Data.TableView view;
			Cerebrum.ObjectHandle[] ohAttributes;
			string[] Rows;
			string[] FieldsNameInCSV;
			char[] chEnter = new char[]{Convert.ToChar(13), Convert.ToChar(10)};

			try
			{
				//    .
				Rows = fi.FileText.Split(chEnter);
				//      *.csv.
				FieldsNameInCSV = Rows[0].Split(new char[]{';'});
				//  .
#warning "     "
				ohAttributes = new Cerebrum.ObjectHandle[FieldsNameInCSV.Length];
				for(int c=0; c<FieldsNameInCSV.Length; c++)
				{
					if(fi.FieldsInfo.Contains(FieldsNameInCSV[c].Trim(), FindFieldContains.FieldNameInCSV) == false)
					{
						ohAttributes[c] = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Attributes", "Name", FieldsNameInCSV[c].Trim());
					}
					else
					{
						string AttrName = fi.FieldsInfo.GetAttributeName(FieldsNameInCSV[c].Trim());
						ohAttributes[c] = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Attributes", "Name", AttrName);
					}
				}
				//  .
				using(view = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlSpecifiers"))
				{
					Cerebrum.ObjectHandle PrimarySpecifier = Cerebrum.ObjectHandle.Null;

					for(int r=1; r<Rows.Length; r++)
					{
						if(Rows[r] != "")
						{
							string[] AttrVals = Rows[r].Split(new char[]{';'});

							Cerebrum.Data.ComponentItemView item = view.AddNew();
							//    .
#warning "    "
							PrimarySpecifier = FillAttributes(fi, item, "ZlSpecifiers", FieldsNameInCSV, ohAttributes, AttrVals);

							//        DupleTableName ,    .
							using (Cerebrum.Data.TableView view1 = Cerebrum.Management.Utilites.GetView(m_DomainContext, "Attributes"))
							{
								view1.EngageConnector(item.ObjectHandle, item.GetComposite());
								Cerebrum.Management.Utilites.RefreshHandle(m_DomainContext, "Attributes", "Name", item.ObjectHandle);
							}
						}
					}
				}
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
		}

		public Support2 IndexStatements(bool grammarSensitive, Cerebrum.Data.TableView viewS, ZlStatement.DomainProperties statementProps, bool makeWordindex)
		{
			Support2 rs = new Support2();
			//  -      .
			System.Collections.Hashtable htErrors = new System.Collections.Hashtable();

			int DuplicateCount = 0;

			//       ZlStatements.
			for(int r1=0; r1<viewS.Count; r1++)
			{
				Cerebrum.Data.ComponentItemView itemS = viewS[r1];
						
				//   ,      .
				IndexStatement(grammarSensitive, rs.htStatements, itemS, ref DuplicateCount, statementProps);

				string SpecName = "";
				string EndWordBase = "";
				string BeginWordBase = itemS["WordBase"].ToString();
				Cerebrum.ObjectHandle transGround;
				//     ZlTransforms.
#warning "untested"
				using(Cerebrum.IComposite compS = itemS.GetComposite())
				using(Cerebrum.IComposite comp1 = ((Cerebrum.IContainer)compS).AttachConnector(statementProps.EndingModelHandle))
				{
					transGround = comp1.GroundHandle;
					ZlTransform tr = (ZlTransform)comp1.Component;
					//          .
					Cerebrum.ObjectHandle h2 = (Cerebrum.ObjectHandle)tr.GetAttributeVal("PrimarySpecifier");
					//     .
					EndWordBase = tr.GetAttributeComponent(h2).ToString();
					//
					using(Cerebrum.IComposite comp2 = m_DomainContext.AttachConnector(h2))
					{
						SpecName = ((ZlSpecifier)comp2.Component).GetAttributeVal("Name").ToString().Trim();
					}
				}
				//  .
				string WordBase = SpecName.Substring(2) + " " + BeginWordBase + " " + EndWordBase;
				string Wordindex = BeginWordBase + EndWordBase;

				if(makeWordindex)
				{
					System.Collections.ArrayList al;
					if(rs.htWordindex.ContainsKey(Wordindex) == false)
					{
						al = new System.Collections.ArrayList();
						rs.htWordindex.Add(Wordindex, al);
					}
					else
					{
						al = (System.Collections.ArrayList)rs.htWordindex[Wordindex];
					}
					al.Add(transGround);
				}

				if(rs.htWordforms.ContainsKey(WordBase) == false)
				{
					rs.htWordforms.Add(WordBase, new Support1(transGround, EndWordBase));
				}
				else
				{
					System.Collections.ArrayList al;
					//    .
					Support1 sp1 = (Support1)rs.htWordforms[WordBase];
					if(htErrors.ContainsKey(WordBase) == false)
					{
						al = new System.Collections.ArrayList();
						htErrors.Add(WordBase, al);
					}
					else
					{
						al = (System.Collections.ArrayList)htErrors[WordBase];
					}
					if(!al.Contains(sp1.m_hTransformGr))
						al.Add(sp1.m_hTransformGr);
					if(!al.Contains(transGround))
						al.Add(transGround);
				}
			}
			if(DuplicateCount > 0)
			{
				throw(new Exception("   = " + DuplicateCount.ToString()));
			}

			//
			System.Collections.ArrayList alReport = new System.Collections.ArrayList();
			foreach(System.Collections.DictionaryEntry de in htErrors)
			{
				string WordBase = de.Key.ToString();
				System.Collections.ArrayList al = (System.Collections.ArrayList)de.Value;
				for(int i=0; i<al.Count; i++)
				{
					Cerebrum.ObjectHandle h2 = (Cerebrum.ObjectHandle)al[i];
					alReport.Add("WordBase = " + WordBase + "; TransformHandle = " + h2.ToString());
				}
			}
			ReportForm dlg = new ReportForm(alReport);
			dlg.ShowDialog();

			//    .  
			foreach(System.Collections.DictionaryEntry de in htErrors)
			{
				string WordBase = de.Key.ToString();
				rs.htWordforms.Remove(WordBase);
			}

			//   
			return rs;
		}

		public void ImportZlTransforms(FileInfo fi, bool grammarSensitive)
		{
			char[] chEnter = new char[]{Convert.ToChar(13), Convert.ToChar(10)};
			ZlStatement.DomainProperties statementProps = ZlStatement.GetProperties(this.m_DomainContext);

			try
			{
				Cerebrum.ObjectHandle PrefixTransTypeHandle = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Types", "Name", "ZlPrefixTransform");
				Cerebrum.ObjectHandle EndingTransTypeHandle = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Types", "Name", "ZlEndingTransform");

				//     ZlSpecifiers    ZlTransforms  .
				//       ,       ZlSpecifiers  ZlTransforms   SelectedComponents  SelectedAttributes
				//   ZlTransforms.
				using (Cerebrum.Data.TableView view1 = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlTransforms"))
				{
					Cerebrum.Reflection.TableDescriptor td = view1.TableDescriptor;
					//   SelectedAttributes.
					using (Cerebrum.Runtime.NativeWarden nw = td.GetSelectedAttributesVector())
					{
						//     ZlSpecifiers    ZlTransforms  .
						using (Cerebrum.Data.TableView view2 = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlSpecifiers"))
						{
							for(int r2=0; r2<view2.Count; r2++)
							{
								using(Cerebrum.IComposite comp2 = view2[r2].GetComposite())
								{
									if(comp2 != null)
									{
										Cerebrum.Integrator.Utilites.EngageConnector(nw, view2[r2].ObjectHandle, comp2, true);
									}
								}
							}
						}
					}
				}

				//    .
				string[] Rows = fi.FileText.Split(chEnter);
				//      *.csv.
				string[] FieldsNameInCSV = Rows[0].Split(new char[]{';'});

				Cerebrum.ObjectHandle ohaWordBase = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Attributes", "Name", "WordBase");
				Cerebrum.ObjectHandle ohaPrimSpec = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Attributes", "Name", "PrimarySpecifier");

				//    .
				int FieldTransformIndex = -1;
				int FieldOrdinalIndex = -1;
				int FieldWordformIndex = -1;
				int FieldSpecNameIndex = -1;
				int FieldFlexiaIndex = -1;
				for(int c=0; c<FieldsNameInCSV.Length; c++)
				{
					if(FieldsNameInCSV[c].Trim() == "Name")
					{
						FieldTransformIndex = c;
					}
					if(FieldsNameInCSV[c].Trim() == "Type")
					{
						FieldOrdinalIndex = c;
					}
					if(FieldsNameInCSV[c].Trim() == "Wordform")
					{
						FieldWordformIndex = c;
					}
					if(FieldsNameInCSV[c].Trim() == "Specifier")
					{
						FieldSpecNameIndex = c;
					}
					if(FieldsNameInCSV[c].Trim() == "SpecName")
					{
						FieldSpecNameIndex = c;
					}
					if(FieldsNameInCSV[c].Trim() == "Flexia")
					{
						FieldFlexiaIndex = c;
					}
				}

				System.Collections.Hashtable htZlTransforms = new System.Collections.Hashtable();
#warning TODO     htZlTransforms   

				//         .
				System.Collections.Hashtable htZlStatements = StatementsHashTable(true, statementProps);

				//   ZlTransforms.
				using(Cerebrum.Data.TableView viewT = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlTransforms"), viewS = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlStatements"))
				{
					string TransformName = null;
					string PrimSpecName = null;
					System.Collections.SortedList FlexiasList = null;
					Cerebrum.ObjectHandle TypeHandle = Cerebrum.ObjectHandle.Null;
					Cerebrum.Vocabulary.Desktop.FlexiaMode FlexiaMode = Cerebrum.Vocabulary.Desktop.FlexiaMode.None;
					string WordBase = null;

					for(int r=1; r<Rows.Length; r++)
					{
						string row = Rows[r];
						try
						{
							if(row!=null && row != string.Empty)
							{
								string[] AttrVals = row.Split(new char[]{';'});
								if(AttrVals.Length < 5)
								{
									throw new System.Exception("Insufficient fields in the row");
								}

								string wordform = AttrVals[FieldWordformIndex];
								string flexia = AttrVals[FieldFlexiaIndex];
								string wordbase = string.Empty;
								string transmode = AttrVals[FieldOrdinalIndex].Trim();

								bool entrance = (TransformName!=AttrVals[FieldTransformIndex].Trim());

								if(entrance)
								{
									if(FlexiasList!=null)
									{
										CreateTransform(TypeHandle, htZlTransforms, htZlStatements, viewT, viewS, FlexiasList, TransformName, PrimSpecName, WordBase, FlexiaMode, statementProps, ohaPrimSpec, ohaWordBase, grammarSensitive);
										FlexiasList = null;
									}

									if(transmode == Cerebrum.Vocabulary.Concepts.KnownValues.EndingMode)
									{
										TypeHandle = EndingTransTypeHandle;
										FlexiaMode = Cerebrum.Vocabulary.Desktop.FlexiaMode.Ending;
									}
									else if(transmode == Cerebrum.Vocabulary.Concepts.KnownValues.PrefixMode)
									{
										TypeHandle = PrefixTransTypeHandle;
										FlexiaMode = Cerebrum.Vocabulary.Desktop.FlexiaMode.Prefix;
									}
									else
									{
										throw new System.Exception("Unknown transform type '" + transmode + "'");
									}
								}

								if(FlexiaMode == Cerebrum.Vocabulary.Desktop.FlexiaMode.Ending)
								{
									if(wordform.EndsWith(flexia))
									{
										wordbase = wordform.Substring(0, wordform.Length - flexia.Length);
									}
									else
									{
										throw new System.Exception("Transform=" + AttrVals[FieldTransformIndex] + "; Specifier=" + AttrVals[FieldSpecNameIndex] + "; Wrong ending '" + wordform + "' '" + flexia + "'");
									}
								}
								if(FlexiaMode == Cerebrum.Vocabulary.Desktop.FlexiaMode.Prefix)
								{
									if(wordform.StartsWith(flexia))
									{
										wordbase = wordform.Substring(flexia.Length);
									}
									else
									{
										throw new System.Exception("Transform=" + AttrVals[FieldTransformIndex] + "; Specifier=" + AttrVals[FieldSpecNameIndex] + "; Wrong prefix '" + flexia + "' '" + wordform + "'");
									}
								}

								if(entrance)
								{
									FlexiasList = new System.Collections.SortedList();
									TransformName = AttrVals[FieldTransformIndex].Trim();
									PrimSpecName = AttrVals[FieldSpecNameIndex].Trim();
									WordBase = wordbase;
								}
								else
								{
									if(transmode!="0")
									{
										throw new System.Exception("Unknown transform type '" + transmode + "'");
									}
									if(WordBase != wordbase)
									{
										throw new System.Exception("Transform=" + AttrVals[FieldTransformIndex] + "; Specifier=" + AttrVals[FieldSpecNameIndex] + "; Wrong base '" + wordform + "' '" + WordBase + "'");
									}
								}

								if(FlexiasList.ContainsKey(AttrVals[FieldSpecNameIndex]))
								{
									throw new System.Exception("Specifier '" + AttrVals[FieldSpecNameIndex] + "' for flexia '" + flexia + "' already exists in transform '" + TransformName + "' with flexia '" + Convert.ToString(FlexiasList[AttrVals[FieldSpecNameIndex]]) + "'");
								}
								else
								{
									FlexiasList.Add(AttrVals[FieldSpecNameIndex], flexia);
								}
							}
						}
						catch(System.Exception ex)
						{
							throw new System.Exception("Failure in row " + (r+1).ToString() + ": '" + row + "'", ex);
						}
					}
					if(FlexiasList!=null)
					{
						CreateTransform(TypeHandle, htZlTransforms, htZlStatements, viewT, viewS, FlexiasList, TransformName, PrimSpecName, WordBase, FlexiaMode, statementProps, ohaPrimSpec, ohaWordBase, grammarSensitive);
					}
				}
				Cerebrum.Management.Utilites.RefreshHandles(m_DomainContext, "ZlTransforms", "Name");
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
		}

		public void ImportZlStatements(FileInfo fi, bool checkFlexia, bool grammarSensitive)
		{
			long TotalWords = 0;
			char[] chEnter = new char[]{Convert.ToChar(13), Convert.ToChar(10)};
			System.Collections.Hashtable htTransforms = new System.Collections.Hashtable();
			string TransformNames = "";

			string appPath = Cerebrum.Windows.Forms.Application.Instance.ApplicationDirectory;
			System.IO.FileStream fslog = new System.IO.FileStream(System.IO.Path.Combine(appPath, @"Export\import.log"), System.IO.FileMode.Create, System.IO.FileAccess.Write, System.IO.FileShare.Read);

			try
			{
				//    .
				string[] Rows = fi.FileText.Split(chEnter);
				for(int i=0; i<Rows.Length; i++)
				{
					string row = Rows[i].Trim();
					if(row!=null && row !=string.Empty)
					{
						System.Collections.ArrayList al;
						//       ...
						if(row.StartsWith("["))
						{
							bool wrong = true;
							if(row.EndsWith("]"))
							{
								//   .
								TransformNames = row.Substring(1, row.Length - 2);
								//       .
								if(htTransforms.ContainsKey(TransformNames) == false)
								{
									al = new System.Collections.ArrayList();
									htTransforms.Add(TransformNames, al);
								}
								wrong = false;
							}
							if(wrong)
							{
								throw new System.Exception("Wrong Statement row = '" + row + "'");
							}
						}
						else
						{
							al = (System.Collections.ArrayList)htTransforms[TransformNames];
							al.Add(row);
						}
					}
				}

				Cerebrum.ObjectHandle specifierAttrHandle = Cerebrum.Management.Utilites.ResolveHandle(this.m_DomainContext, "Attributes", "Name", "PrimarySpecifier");
				ZlStatement.DomainProperties statementProps = ZlStatement.GetProperties(this.m_DomainContext);

				System.Collections.Hashtable htStatements = StatementsHashTable(grammarSensitive, statementProps);

				using(Cerebrum.Data.TableView viewS = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlStatements"))
				{
					foreach(System.Collections.DictionaryEntry de in htTransforms)
					{
						//   .

						string []TransformParts = Convert.ToString(de.Key).Split(';');
						string sPrefixModel;
						string sEndingModel;
						if(TransformParts.Length ==1)
						{
							sPrefixModel = string.Empty;
							sEndingModel = TransformParts[0];
						}
						else if(TransformParts.Length ==2)
						{
							sPrefixModel = TransformParts[0];
							sEndingModel = TransformParts[1];
						}
						else
						{
							throw new System.Exception("Wrong transform names = '" + TransformNames + "'");
						}

						//     .
						if(htTransformMap.ContainsKey(sPrefixModel) == true)
						{
							sPrefixModel = htTransformMap[sPrefixModel].ToString();
						}
						if(htTransformMap.ContainsKey(sEndingModel) == true)
						{
							sEndingModel = htTransformMap[sEndingModel].ToString();
						}
						TransformNames = sPrefixModel + ";" + sEndingModel;

						Cerebrum.ObjectHandle hPrefixModel = Cerebrum.ObjectHandle.Null;
						Cerebrum.ObjectHandle hEndingModel = Cerebrum.ObjectHandle.Null;
						if(sPrefixModel!=null && sPrefixModel!=string.Empty)
						{
							hPrefixModel = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "ZlTransforms", "Name", sPrefixModel);
							if(hPrefixModel==Cerebrum.ObjectHandle.Null)
							{
								throw new Exception("Prefix Model Transform name='" + sPrefixModel + "' is absent");
							}
						}
						if(sEndingModel!=null && sEndingModel!=string.Empty)
						{
							hEndingModel = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "ZlTransforms", "Name", sEndingModel);
							if(hEndingModel==Cerebrum.ObjectHandle.Null)
							{
								throw new Exception("Ending Model Transform name='" + sEndingModel + "' is absent");
							}
						}

						Cerebrum.IComposite compPrefixModel = null;
						Cerebrum.IComposite compEndingModel = null;
						if(!Cerebrum.ObjectHandle.Null.Equals(hPrefixModel))
						{
							compPrefixModel = m_DomainContext.AttachConnector(hPrefixModel);
						}
						if(!Cerebrum.ObjectHandle.Null.Equals(hEndingModel))
						{
							compEndingModel = m_DomainContext.AttachConnector(hEndingModel);
						}
						try
						{
							byte [] temp = System.Text.Encoding.Default.GetBytes("[" + TransformNames + "] " + TotalWords.ToString() + System.Environment.NewLine);
							fslog.Write(temp, 0, temp.Length);fslog.Flush();

							string prefix = string.Empty;
							string ending = string.Empty;

							if(compPrefixModel!=null)
							{
								//    PrimarySpecifier   ZlTransforms   .
								Cerebrum.ObjectHandle prefixSpecifierHandle = (Cerebrum.ObjectHandle)Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)compPrefixModel, specifierAttrHandle);
								//   .
								prefix = Convert.ToString(Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)compPrefixModel, prefixSpecifierHandle));
							}

							if(compEndingModel!=null)
							{
								//    PrimarySpecifier   ZlTransforms   .
								Cerebrum.ObjectHandle endingSpecifierHandle = (Cerebrum.ObjectHandle)Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)compEndingModel, specifierAttrHandle);
								//   .
								ending = Convert.ToString(Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)compEndingModel, endingSpecifierHandle));
							}

							//      .
							System.Collections.ArrayList al = (System.Collections.ArrayList)de.Value;
							al.Sort();
							for(int i2=0; i2<al.Count; i2++)
							{
								string wordForm = al[i2].ToString();

								//     .
								string wordBase = VerifyStatement(checkFlexia, prefix, wordForm, ending, TransformNames);
								CreateStatement(htStatements, viewS, wordBase, TransformNames, compPrefixModel, compEndingModel, statementProps, grammarSensitive);

								temp = System.Text.Encoding.Default.GetBytes(wordForm + " " + ending + System.Environment.NewLine);
								fslog.Write(temp, 0, temp.Length);fslog.Flush();

								TotalWords++;
							}
						}
						finally
						{
							if(compPrefixModel!=null)
							{
								compPrefixModel.Dispose();
							}
							if(compEndingModel!=null)
							{
								compEndingModel.Dispose();
							}
						}
					}
				}
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
			fslog.Close();
		}

		public Cerebrum.Data.ComponentItemView CheckSpecifierEndings(Cerebrum.Data.TableView view, System.Collections.SortedList transPairs, System.Collections.Hashtable htZlTransforms, Cerebrum.ObjectHandle TypeHandle, out bool isNew)
		{
			Cerebrum.Data.ComponentItemView item;
			string sList = "";
			for(int i = 0; i < transPairs.Count; i++)
			{
				string AttrName = transPairs.GetKey(i).ToString().Trim();
				string AttrVal = transPairs.GetByIndex(i).ToString().Trim();
				if(AttrName != string.Empty)
				{
					if(AttrVal == "")
					{
						AttrVal = "()";
					}
					sList += AttrName + "=" + AttrVal + ";";
				}
			}
			if(htZlTransforms.ContainsKey(sList) == false)
			{
				//     .
				Cerebrum.ObjectHandle NextObjectHandle = m_DomainContext.NextSequence();
				//   .
				item = view.AddNew(NextObjectHandle, TypeHandle);
				htZlTransforms.Add(sList, item.ObjectHandle);
				isNew = true;
			}
			else
			{
				item = view.FindItemViewByObjectHandle((Cerebrum.ObjectHandle)htZlTransforms[sList]);
				isNew = false;
			}
			return item;
		}

		public void CreateTransform(Cerebrum.ObjectHandle TypeHandle, System.Collections.Hashtable htZlTransforms, System.Collections.Hashtable htZlStatements, Cerebrum.Data.TableView viewT, Cerebrum.Data.TableView viewS, System.Collections.SortedList transPairs, string transformName, string primSpecifier, string wordBase, Cerebrum.Vocabulary.Desktop.FlexiaMode flexiaMode, ZlStatement.DomainProperties statementProps, Cerebrum.ObjectHandle ohaPrimSpec, Cerebrum.ObjectHandle ohaWordBase, bool grammarSensitive)
		{
			bool isNew;
			//     .
			Cerebrum.Data.ComponentItemView itemT = CheckSpecifierEndings(viewT, transPairs, htZlTransforms, TypeHandle, out isNew);
			using(Cerebrum.IComposite compTransform = itemT.GetComposite())
			{
				if(isNew)
				{
					Cerebrum.Integrator.Utilites.UpdateComponent((Cerebrum.IContainer)compTransform, Cerebrum.Specialized.Concepts.NameAttribute, transformName);

					Cerebrum.ObjectHandle ohPrimSpecInstance = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Attributes", "Name", primSpecifier);
					Cerebrum.Integrator.Utilites.UpdateComponent((Cerebrum.IContainer)compTransform, ohaPrimSpec, ohPrimSpecInstance);
					/*
					 * Direct JOIN can't be used because Handle is needed to get ending quickly
					 * 
					using(Cerebrum.IConnector connector = this.m_DomainContext.AttachConnector(ohPrimSpecInstance))
					{
						((Cerebrum.IContainer)compTransform).EngageConnector(ohaPrimSpec, connector);
					}*/

					//EngageSpecifierEndings(compTransform, transPairs);
					for(int i = 0; i < transPairs.Count; i++)
					{
						string AttrName = transPairs.GetKey(i).ToString().Trim();
						string AttrVal = transPairs.GetByIndex(i).ToString().Trim();
						Cerebrum.ObjectHandle AttrHandle = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "Attributes", "Name", AttrName);
						using(Cerebrum.IComposite compText = Cerebrum.Vocabulary.Desktop.Utilites.AttachStringQuoted(this.m_WMaster, AttrVal, false))
						{
							((Cerebrum.IContainer)compTransform).EngageConnector(AttrHandle, compText);
							if(flexiaMode==Cerebrum.Vocabulary.Desktop.FlexiaMode.Prefix)
							{
								EngageComponent(compText, compTransform);
							}
						}
					}

					if(wordBase!=null && wordBase!=string.Empty)
					{
						using(Cerebrum.IComposite compText = Cerebrum.Vocabulary.Desktop.Utilites.AttachStringQuoted(this.m_WMaster, wordBase, false))
						{
							((Cerebrum.IContainer)compTransform).EngageConnector(ohaWordBase, compText);
						}
					}
				}
				else
				{
					//      
					//     .
					string NewTransName = Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)compTransform, Cerebrum.Specialized.Concepts.NameAttribute).ToString();
					htTransformMap.Add(transformName, NewTransName);
				}
				string TransformNames=";" + Convert.ToString(compTransform.Component);
				if(wordBase!=null && flexiaMode==Cerebrum.Vocabulary.Desktop.FlexiaMode.Ending)
				{
					CreateStatement(htZlStatements, viewS, wordBase, TransformNames, null, compTransform, statementProps, grammarSensitive);
				}
			}
		}

		public string VerifyStatement(bool checkFlexia, string prefix, string wordform, string ending, string transformNames)
		{
			if(transformNames=="11893;11774")
			{
				System.Diagnostics.Debug.WriteLine("11893;11774");
			}
			string wordBase = wordform;
			if(checkFlexia == true)
			{
				if(ending!=null && ending!=string.Empty)
				{
					if(wordBase.EndsWith(ending))
					{
						wordBase = wordBase.Substring(0, wordBase.Length - ending.Length);
					}
					else
					{
						throw new System.Exception("Transform '" + transformNames + "' wrong ending '" + wordform + "' '" + ending + "'");
					}
				}
				if(prefix!=null && prefix!=string.Empty)
				{
					if(wordBase.StartsWith(prefix))
					{
						wordBase = wordBase.Substring(prefix.Length);
					}
					else
					{
						throw new System.Exception("Transform '" + transformNames + "' wrong prefix '" + prefix + "' '" + wordform + "'");
					}
				}
			}
			return wordBase;
		}

		private void EngageComponent(Cerebrum.IComposite compText, Cerebrum.IComposite compItem)
		{
			if(compText is Cerebrum.IContainer)
			{
				((Cerebrum.IContainer)compText).EngageConnector(compItem.PlasmaHandle, compItem);
			}
			else if(compText is System.IO.Stream)
			{
				System.IO.Stream stm = compText as System.IO.Stream;
				Cerebrum.Collections.Specialized.HandleCollection cll;
				cll = new Cerebrum.Collections.Specialized.HandleCollection();
				if(stm.Length > 0)
				{
					stm.Seek(0, System.IO.SeekOrigin.Begin);
					cll.Serialize(Cerebrum.SerializeDirection.Load, compText, stm);
				}
				cll.Add(compItem.PlasmaHandle);
				stm.Seek(0, System.IO.SeekOrigin.Begin);
				cll.Serialize(Cerebrum.SerializeDirection.Save, compText, stm);
				stm.SetLength(stm.Position);
			}
		}

		public void CreateStatement(System.Collections.Hashtable htStatements, Cerebrum.Data.TableView viewS, string wordBase, string transformNames, Cerebrum.IComposite compPrefixModel, Cerebrum.IComposite compEndingModel, ZlStatement.DomainProperties statementProps, bool grammarSensitive)
		{
			//        ...
			string statementKey;
			if(grammarSensitive)
			{
				statementKey = wordBase + transformNames;
			}
			else
			{
				statementKey = wordBase.Replace("(", "").Replace(")", "") + transformNames;
			}
			if(htStatements.ContainsKey(statementKey) == false)
			{
				int DuplicateCount = 0;
				Cerebrum.Data.ComponentItemView item = viewS.AddNew();
				using(Cerebrum.IComposite compItem = item.GetComposite())
				{
					//System.Collections.Hashtable hshOpt = new System.Collections.Hashtable();
					using(Cerebrum.IComposite compText = Cerebrum.Vocabulary.Desktop.Utilites.AttachStringQuoted(this.m_WMaster, wordBase, false))
					{
						((Cerebrum.IContainer)compItem).EngageConnector(statementProps.WordBaseHandle, compText);

						EngageComponent(compText, compItem);
					}

					if(compPrefixModel!=null)
					{
						((Cerebrum.IContainer)compItem).EngageConnector(statementProps.PrefixModelHandle, compPrefixModel);
					}
					if(compEndingModel!=null)
					{
						((Cerebrum.IContainer)compItem).EngageConnector(statementProps.EndingModelHandle, compEndingModel);
					}
				}
				IndexStatementCore(htStatements, statementKey, item.ObjectHandle, ref DuplicateCount);
			}
		}

		/// <summary>
		///   ,      .
		/// </summary>
		/// <returns></returns>
		private System.Collections.Hashtable StatementsHashTable(bool grammarSensitive, ZlStatement.DomainProperties statementProps)
		{
			System.Collections.Hashtable res = new System.Collections.Hashtable();
			int DuplicateCount = 0;

			using (Cerebrum.Data.TableView view1 = Cerebrum.Management.Utilites.GetView(m_DomainContext, "ZlStatements"))
			{
				for(int r=0; r<view1.Count; r++)
				{
					IndexStatement(grammarSensitive, res, view1[r], ref DuplicateCount, statementProps);
				}
			}
			if(DuplicateCount > 0)
			{
				throw(new Exception("   = " + DuplicateCount.ToString()));
			}
			return res;
		}

		private void IndexStatement(bool grammarSensitive, System.Collections.Hashtable htStatements, Cerebrum.Data.ComponentItemView itemS, ref int DuplicateCount, ZlStatement.DomainProperties statementProps)
		{
			string key = string.Empty;
			using(Cerebrum.IComposite compS = itemS.GetComposite())
			{
				string wordBase = string.Empty;

				using(Cerebrum.IComposite compW = ((Cerebrum.IContainer)compS).AttachConnector(statementProps.WordBaseHandle))
				{
					if(grammarSensitive && (this.m_WMaster is Cerebrum.Runtime.HFacade))
					{
						wordBase = Utilites.FilterNodeText(((Cerebrum.Runtime.HFacade)this.m_WMaster).ObtainText(compW));
					}
					else
					{
						wordBase = Convert.ToString(compW.Component);
					}
				}

				string prefixModelName = string.Empty;
				string endingModelName = string.Empty;
				using(Cerebrum.IComposite container = ((Cerebrum.IContainer)compS).AttachConnector(statementProps.PrefixModelHandle))
				{
					if(container!=null)
					{
						prefixModelName = Convert.ToString(Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)container, Cerebrum.Specialized.Concepts.NameAttribute));
					}
				}
				using(Cerebrum.IComposite container = ((Cerebrum.IContainer)compS).AttachConnector(statementProps.EndingModelHandle))
				{
					if(container!=null)
					{
						endingModelName = Convert.ToString(Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)container, Cerebrum.Specialized.Concepts.NameAttribute));
					}
				}
				key = wordBase + prefixModelName + ";" + endingModelName;
			}

			IndexStatementCore(htStatements, key, itemS.ObjectHandle, ref DuplicateCount);
		}

		private void IndexStatementCore(System.Collections.Hashtable htStatements, string statementKey, Cerebrum.ObjectHandle statementHandle, ref int DuplicateCount)
		{
			System.Collections.ArrayList alStatements = null;
			
			if(htStatements.ContainsKey(statementKey) == false)
			{
				alStatements = new System.Collections.ArrayList();
				htStatements.Add(statementKey, alStatements);
			}
			else
			{
				alStatements = (System.Collections.ArrayList)htStatements[statementKey];
				DuplicateCount++;
			}
			alStatements.Add(statementHandle);
		}

		/// <summary>
		///       .
		/// </summary>
		/// <param name="fi">FileInfo</param>
		/// <param name="item">  .</param>
		/// <param name="TableName"> .</param>
		/// <param name="FieldsNameInCSV">     *.csv</param>
		/// <param name="ohAttributes"> ObjectHandle     *.csv</param>
		/// <param name="AttrVal"> ,       .</param>
		/// <returns>ObjectHandle      Ordinal = 0   ZlTransforms.csv.      ZlTransforms.csv.</returns>
		private Cerebrum.ObjectHandle FillAttributes(FileInfo fi, Cerebrum.Data.ComponentItemView item, string TableName, string[] FieldsNameInCSV, Cerebrum.ObjectHandle[] ohAttributes, string[] AttrVal)
		{
			Cerebrum.ObjectHandle PrimarySpecifier = Cerebrum.ObjectHandle.Null;

			try
			{
				using(Cerebrum.IComposite compItem = item.GetComposite())
				{
					// ---===   ZlSpecifiers. ===---
					if(TableName == "ZlSpecifiers")
					{
						Cerebrum.Reflection.AttributeDescriptor ad = (Cerebrum.Reflection.AttributeDescriptor)compItem.Component;
						ad.AttributeTypeIdHandle = new ObjectHandle(1010);
					}
					// ------------------------------------------

					Cerebrum.IContainer con = compItem as Cerebrum.IContainer;
					for(int c=0; c<AttrVal.Length; c++)
					{
						// ---===    ZlTransforms.csv ===---
						int Ordinal = -1;
						if(FieldsNameInCSV[c].Trim() == "Ordinal")
						{
							Ordinal = Convert.ToInt32(AttrVal[c]);
							if(Ordinal != 0)
							{
								PrimarySpecifier = item.ObjectHandle;
							}
							item["PrimarySpecifier"] = PrimarySpecifier;
						}
						// ---------------------------------------------------

						FieldInfo vi = fi.FieldsInfo.FindFieldInfo(FieldsNameInCSV[c].Trim(), FindFieldContains.FieldNameInCSV);
						bool IsVector = false;
						bool IsLoad = true;
						if(vi != null)
						{
							if(vi.WayOfDataRecording == WayOfDataRecording.Vector)
							{
								IsVector = true;
							}
							IsLoad = vi.IsLoad;
						}
						//        ...
						if(IsLoad == true)
						{
							//      ...
							if(IsVector == false)
							{
								if(ohAttributes[c] != Cerebrum.ObjectHandle.Null)
								{
									//      .
									bool Standard = false;
									//      ...
									if(vi != null)
									{
										//     ...
										if(vi.WayOfDataRecording == WayOfDataRecording.Standard)
										{
											//  .
											Standard = true;
										}
									}
									//   Name     .
									if(ohAttributes[c] == new Cerebrum.ObjectHandle(1000))
									{
										Standard = true;
									}

									if(Standard == true)
									{
										string AttrName = fi.FieldsInfo.GetAttributeName(FieldsNameInCSV[c].Trim());
										if(AttrName == "")
										{
											AttrName = FieldsNameInCSV[c].Trim();
										}
										string AttrValue = AttributeValue(fi, TableName, c, FieldsNameInCSV, AttrVal);
										//   .
										item[AttrName] = AttrValue;
									}
									else
									{
										string AttrValue = AttributeValue(fi, TableName, c, FieldsNameInCSV, AttrVal);
										//    .
										using(Cerebrum.IComposite compText = Cerebrum.Vocabulary.Desktop.Utilites.AttachStringQuoted(this.m_WMaster, AttrValue, false))
										{
											con.EngageConnector(ohAttributes[c], compText);
										}
									}
								}
							}
							else
							{
								//   ,     .
								string[] VectorVals = AttrVal[c].Split(new char[]{','});

								//  .
								if(compItem.Component.GetType() == typeof(Cerebrum.Vocabulary.ZlSpecifier))
								{
									Cerebrum.Vocabulary.ZlSpecifier sp = (Cerebrum.Vocabulary.ZlSpecifier)compItem.Component;
									//     .
									for(int i=0; i<VectorVals.Length; i++)
									{
										//  ObjectHandle    Name   ZlAttributes.
										Cerebrum.ObjectHandle h = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "ZlAttributes", "Name", VectorVals[i].Trim());
										//     .
										string AttrName = fi.FieldsInfo.GetAttributeName(FieldsNameInCSV[c].Trim());
										if(AttrName == "")
										{
											AttrName = FieldsNameInCSV[c].Trim();
										}
										if(h == Cerebrum.ObjectHandle.Null)
										{
											string Message = ": ImportCSV.cs. : FillAttributes(). ";
											Message += "       '";
											Message += VectorVals[i].Trim() + "'.";
											throw(new Exception(Message));
										}
										sp.AddChildrenNode(AttrName, h);
									}
								}
								else
								{
									Cerebrum.Vocabulary.BaseNeuron bn = (Cerebrum.Vocabulary.BaseNeuron)compItem.Component;
									if(bn != null)
									{
										//     .
										for(int i=0; i<VectorVals.Length; i++)
										{
											//  ObjectHandle    Name   ZlAttributes.
											Cerebrum.ObjectHandle h = Cerebrum.Management.Utilites.ResolveHandle(m_DomainContext, "ZlAttributes", "Name", VectorVals[i].Trim());
											//     .
											string AttrName = fi.FieldsInfo.GetAttributeName(FieldsNameInCSV[c].Trim());
											if(AttrName == "")
											{
												AttrName = FieldsNameInCSV[c].Trim();
											}
											bn.AddChildrenNode(AttrName, h);
										}
									}
								}
							}
						}
					}
					Cerebrum.Management.Utilites.RefreshHandle(m_DomainContext, TableName, "Name", item.ObjectHandle);
				}
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
			return PrimarySpecifier;
		}

		/// <summary>
		///   ,       .
		/// </summary>
		/// <param name="fi"></param>
		/// <param name="TableName"></param>
		/// <param name="CurrentFieldIndex"></param>
		/// <param name="FieldsNameInCSV"></param>
		/// <param name="AttrsVal"></param>
		/// <returns></returns>
		private string AttributeValue(FileInfo fi, string TableName, int CurrentFieldIndex, string[] FieldsNameInCSV, string[] AttrsVal)
		{
			string AttrValue = "";

			try
			{
				int WordformIndex = -1;
				int EndingIndex = -1;
				for(int c=0; c<FieldsNameInCSV.Length; c++)
				{
					if(FieldsNameInCSV[c].Trim() == "Wordform")
					{
						WordformIndex = c;
					}
					if(FieldsNameInCSV[c].Trim() == "Ending")
					{
						EndingIndex = c;
					}
				}

				AttrValue = AttrsVal[CurrentFieldIndex];
				string AttrName = fi.FieldsInfo.GetAttributeName(FieldsNameInCSV[CurrentFieldIndex].Trim());
				if(AttrName == "")
				{
					AttrName = FieldsNameInCSV[CurrentFieldIndex].Trim();
				}
				if((TableName == "ZlSpecifiers")&&(AttrName == "Description"))
				{
					int n = AttrValue.IndexOf("|");
					if(n == -1)
					{
						AttrValue = "";
					}
					else
					{
						AttrValue = AttrValue.Substring(n + 1);
					}
				}
				if((TableName == "ZlTransforms")&&(AttrName == "WordBase"))
				{
					string WordformVal = AttrsVal[WordformIndex];
					string EndingVal = AttrsVal[EndingIndex];
					if(EndingVal != "" && WordformVal != "" )
					{
						if(WordformVal.EndsWith(EndingVal))
						{
							AttrValue = WordformVal.Substring(0, WordformVal.Length - EndingVal.Length);
						}
						else
						{
							throw new System.Exception("Wrong ending " + WordformVal + " " + EndingVal);
						}
					}
					else
					{
						AttrValue = WordformVal;
					}
				}
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
			return AttrValue;
		}
	}
}
