/******************************************************************************************************

 Cerebrum.Vocabulary.Desktop.dll is part of the Morphology .NET toolkit and contains implementation of
 the toolkit user interface for the OOKBMS Cerebrum desktop environment.

 Copyright (C) 2007-2009 Shuklin Dmitry Evgenievich and Boyko Sergey Vasilievich. All rights reserved.

 The Morphology .NET toolkit is free software; you can redistribute it and/or modify it under the terms 
 of the Microsoft Public License (Ms-PL) as published by the Microsoft.

 This toolkit is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
 QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE 
 COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

 See the Microsoft Public License (Ms-PL) for more details.

 You should have received a copy of the Microsoft Public License (Ms-PL) along with this toolkit;
 if not, the Microsoft Public License (Ms-PL) can be found at 
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL

 Please see the LICENSE.txt file for the full license.

 For all inquieries and concerns contact an authors at  cerebrum@shdsoft.ru

 This copyright notice MUST APPEAR in all copies of this file!

 //////////////////////////////////////////////////////////////////////////////////////////////////////

  Cerebrum.Vocabulary.Desktop.dll -   Morphology .NET toolkit  
          Cerebrum.

 Copyright (C) 2007-2009       .   .

      ;     / 
      Microsoft Public License (Ms-PL).

     ,    ,   - ;
           .  
          .    
 ,        ,   
 .

      Microsoft Public License (Ms-PL)

      Microsoft Public License (Ms-PL)    ; 
   ,  Microsoft Public License (Ms-PL)      
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

       LICENSE.txt

          : cerebrum@shdsoft.ru
 
          !
 
******************************************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace Cerebrum.Vocabulary
{
	/// <summary>
	/// Summary description for ControlPanel.
	/// </summary>
	public class VocabularyDatabaseForm : Cerebrum.Windows.Forms.DesktopComponent
	{
		private class PrefixAndEndingTransforms
		{
			public ForExportWords PrefixTransInfo;
			public ForExportWords EndingTransInfo;

			public PrefixAndEndingTransforms()
			{
				PrefixTransInfo = null;
				EndingTransInfo = null;
			}
		}

		private class ForExportWords
		{
			public string PrimarySpecFlexia;
			public string TransformName;
			public System.Collections.SortedList FlexiaList;

			public ForExportWords()
			{
				PrimarySpecFlexia = "";
				TransformName = "";
				FlexiaList = null;
			}
		}

		private class Flexias
		{
			public string Prefix;
			public string Ending;

			public Flexias()
			{
				Prefix = "";
				Ending = "";
			}
		}

		private class PrefixAndEndingTransformsComparer : System.Collections.IComparer
		{
			#region IComparer Members

			public int Compare(object x, object y)
			{
				string aptn = string.Empty;
				string bptn = string.Empty;
				string aetn = string.Empty;
				string betn = string.Empty;
				PrefixAndEndingTransforms a = null;
				PrefixAndEndingTransforms b = null;

				if(x != null)
				{
					a = (PrefixAndEndingTransforms)x;
					aptn = a.PrefixTransInfo.TransformName;
					bptn = b.PrefixTransInfo.TransformName;
				}
				if(y != null)
				{
					b = (PrefixAndEndingTransforms)y;
					aetn = a.EndingTransInfo.TransformName;
					betn = b.EndingTransInfo.TransformName;
				}
				int p = string.Compare(aptn, bptn);
				int e = string.Compare(aetn, betn);
				return p + e;
				//return string.Compare(p.ToString(), e.ToString());
			}

			#endregion
		}

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Label lblMessage;
		private System.Windows.Forms.OpenFileDialog openFileDialog1;
		private System.Windows.Forms.SaveFileDialog saveFileDialog1;

		private Cerebrum.Data.TableView m_TableView;
		public Cerebrum.Runtime.ISemiotics m_WMaster = null;
		public System.Windows.Forms.DataGrid m_DataGrid1;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.TextBox m_TextBox1;
		private System.Windows.Forms.Button m_btnTransform;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.Button btnMorphAnalysis;
		private System.Windows.Forms.Button btnLemmatize;
		private System.Windows.Forms.ListBox m_ListBox1;

		public VocabularyDatabaseForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			m_TableView = null;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(m_TableView != null)
				{
					m_TableView.Dispose();
				}
				if(components != null)
				{
					components.Dispose();
				}
				if(this.m_WMaster!=null)
				{
					this.m_WMaster.Dispose();
					this.m_WMaster = null;
				}
				if(this.m_SecondDomain!=null)
				{
					m_SecondDomain.Shutdown();
				}
				if(this.m_Connector!=null)
				{
					Cerebrum.Integrator.DomainContext domainContext = this.DomainContext;
					this.m_Connector = null;
					domainContext.Shutdown();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.lblMessage = new System.Windows.Forms.Label();
			this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
			this.saveFileDialog1 = new System.Windows.Forms.SaveFileDialog();
			this.m_DataGrid1 = new System.Windows.Forms.DataGrid();
			this.m_ListBox1 = new System.Windows.Forms.ListBox();
			this.panel1 = new System.Windows.Forms.Panel();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.m_btnTransform = new System.Windows.Forms.Button();
			this.m_TextBox1 = new System.Windows.Forms.TextBox();
			this.label1 = new System.Windows.Forms.Label();
			this.btnMorphAnalysis = new System.Windows.Forms.Button();
			this.btnLemmatize = new System.Windows.Forms.Button();
			((System.ComponentModel.ISupportInitialize)(this.m_DataGrid1)).BeginInit();
			this.panel1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// lblMessage
			// 
			this.lblMessage.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.lblMessage.Location = new System.Drawing.Point(0, 453);
			this.lblMessage.Name = "lblMessage";
			this.lblMessage.Size = new System.Drawing.Size(920, 24);
			this.lblMessage.TabIndex = 4;
			this.lblMessage.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// m_DataGrid1
			// 
			this.m_DataGrid1.DataMember = "";
			this.m_DataGrid1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.m_DataGrid1.HeaderForeColor = System.Drawing.SystemColors.ControlText;
			this.m_DataGrid1.Location = new System.Drawing.Point(200, 0);
			this.m_DataGrid1.Name = "m_DataGrid1";
			this.m_DataGrid1.Size = new System.Drawing.Size(720, 453);
			this.m_DataGrid1.TabIndex = 8;
			// 
			// m_ListBox1
			// 
			this.m_ListBox1.IntegralHeight = false;
			this.m_ListBox1.Items.AddRange(new object[] {
															"ZlAttributes",
															"ZlSpecifiers",
															"ZlTransforms",
															"ZlStatements"});
			this.m_ListBox1.Location = new System.Drawing.Point(8, 16);
			this.m_ListBox1.Name = "m_ListBox1";
			this.m_ListBox1.Size = new System.Drawing.Size(168, 80);
			this.m_ListBox1.TabIndex = 9;
			this.m_ListBox1.SelectedIndexChanged += new System.EventHandler(this.m_ListBox1_SelectedIndexChanged);
			// 
			// panel1
			// 
			this.panel1.Controls.Add(this.groupBox2);
			this.panel1.Controls.Add(this.groupBox1);
			this.panel1.Dock = System.Windows.Forms.DockStyle.Left;
			this.panel1.Location = new System.Drawing.Point(0, 0);
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size(200, 453);
			this.panel1.TabIndex = 10;
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.Add(this.m_ListBox1);
			this.groupBox2.Location = new System.Drawing.Point(8, 336);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(184, 104);
			this.groupBox2.TabIndex = 11;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Tables list";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.btnLemmatize);
			this.groupBox1.Controls.Add(this.btnMorphAnalysis);
			this.groupBox1.Controls.Add(this.m_btnTransform);
			this.groupBox1.Controls.Add(this.m_TextBox1);
			this.groupBox1.Controls.Add(this.label1);
			this.groupBox1.Location = new System.Drawing.Point(8, 8);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(184, 144);
			this.groupBox1.TabIndex = 10;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Test 1";
			// 
			// m_btnTransform
			// 
			this.m_btnTransform.Location = new System.Drawing.Point(8, 64);
			this.m_btnTransform.Name = "m_btnTransform";
			this.m_btnTransform.Size = new System.Drawing.Size(168, 23);
			this.m_btnTransform.TabIndex = 2;
			this.m_btnTransform.Text = "Transform";
			this.m_btnTransform.Click += new System.EventHandler(this.m_btnTransform_Click);
			// 
			// m_TextBox1
			// 
			this.m_TextBox1.Location = new System.Drawing.Point(8, 40);
			this.m_TextBox1.Name = "m_TextBox1";
			this.m_TextBox1.Size = new System.Drawing.Size(168, 20);
			this.m_TextBox1.TabIndex = 1;
			this.m_TextBox1.Text = "";
			this.m_TextBox1.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.m_TextBox1_KeyPress);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 16);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(64, 16);
			this.label1.TabIndex = 0;
			this.label1.Text = "WordBase:";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// btnMorphAnalysis
			// 
			this.btnMorphAnalysis.Location = new System.Drawing.Point(8, 88);
			this.btnMorphAnalysis.Name = "btnMorphAnalysis";
			this.btnMorphAnalysis.Size = new System.Drawing.Size(168, 23);
			this.btnMorphAnalysis.TabIndex = 3;
			this.btnMorphAnalysis.Text = "Morphological Analysis";
			this.btnMorphAnalysis.Click += new System.EventHandler(this.btnMorphAnalysis_Click);
			// 
			// btnLemmatize
			// 
			this.btnLemmatize.Location = new System.Drawing.Point(8, 112);
			this.btnLemmatize.Name = "btnLemmatize";
			this.btnLemmatize.Size = new System.Drawing.Size(168, 23);
			this.btnLemmatize.TabIndex = 4;
			this.btnLemmatize.Text = "Lemmatize";
			this.btnLemmatize.Click += new System.EventHandler(this.btnLemmatize_Click);
			// 
			// VocabularyDatabaseForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(920, 477);
			this.Controls.Add(this.m_DataGrid1);
			this.Controls.Add(this.panel1);
			this.Controls.Add(this.lblMessage);
			this.Name = "VocabularyDatabaseForm";
			this.Text = "Cerebrum Vocabulary ";
			this.Load += new System.EventHandler(this.VocabularyDatabaseForm_Load);
			((System.ComponentModel.ISupportInitialize)(this.m_DataGrid1)).EndInit();
			this.panel1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		Cerebrum.Integrator.DomainContext m_SecondDomain = null;

		private void VocabularyDatabaseForm_Load(object sender, System.EventArgs e)
		{
			int StatCount = 0;

			try
			{
				//   .
				using(Cerebrum.Data.TableView tvStat = Cerebrum.Management.Utilites.GetView(this.DomainContext, "ZlStatements"))
				{
					//     .
					StatCount = tvStat.Count;
				}
				SetStatusBarMessage("   : " + StatCount.ToString());
				this.Text = this.DomainContext.FileName;
			}
			catch (Exception ex)
			{
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
		}

		public void PreInit()
		{
			using(Cerebrum.Runtime.NativeSector sector = (Cerebrum.Runtime.NativeSector)this.m_Connector.Workspace.GetSector())
			{
				sector.Newobj(new Cerebrum.ObjectHandle(60002), Cerebrum.Runtime.KernelObjectKindId.Scalar);
			}

			this.m_WMaster = Cerebrum.Runtime.HFacade.CreateInstance(this.m_Connector.Workspace);

			Cerebrum.ObjectHandle h = this.m_WMaster.CreateSector();
			using(Cerebrum.IConnector connector = this.m_Connector.Workspace.AttachConnector(new Cerebrum.ObjectHandle(60002)))
			{
				connector.Component = h;
			}

			string s = string.Empty;
			s += "MH = " + h.ToString() + System.Environment.NewLine;
		}

		public void PreOpen()
		{
			string s = string.Empty;
			Cerebrum.ObjectHandle hL = Cerebrum.ObjectHandle.Null;
			Cerebrum.ObjectHandle hH = Cerebrum.ObjectHandle.Null;
			using(Cerebrum.IConnector connector = this.m_Connector.Workspace.AttachConnector(new Cerebrum.ObjectHandle(60001)))
			{
				if(connector!=null)
				{
					hL = (Cerebrum.ObjectHandle)connector.Component;
				}
			}
			using(Cerebrum.IConnector connector = this.m_Connector.Workspace.AttachConnector(new Cerebrum.ObjectHandle(60002)))
			{
				if(connector!=null)
				{
					hH = (Cerebrum.ObjectHandle)connector.Component;
				}
			}

			if(hL!=Cerebrum.ObjectHandle.Null)
			{
				this.m_WMaster = Cerebrum.Runtime.LFacade.CreateInstance(this.m_Connector.Workspace);

				this.m_WMaster.ReviveSector(hL);

				s += "ML = " + hL.ToString(); // + System.Environment.NewLine;
			}
			else if(hH!=Cerebrum.ObjectHandle.Null)
			{
				this.m_WMaster = Cerebrum.Runtime.HFacade.CreateInstance(this.m_Connector.Workspace);

				this.m_WMaster.ReviveSector(hH);

				s += "MH = " + hH.ToString(); // + System.Environment.NewLine;
			}
		}

		protected override void SetConnector(SerializeDirection direction, IConnector connector)
		{
			base.SetConnector (direction, connector);
			this.DomainContext.Disposing += new EventHandler(DomainContext_Disposing);
		}

		private void DomainContext_Disposing(object sender, EventArgs e)
		{
			if(this.m_Connector!=null)
			{
				this.m_Connector = null;
				this.Dispose();
			}
		}

		private Cerebrum.IComposite AttachString(string text)
		{
			Cerebrum.ObjectHandle [] ohs = this.m_WMaster.ParseToken(text, false);
			if(ohs!=null && ohs.Length > 0)
			{
				return this.m_WMaster.AttachConnector(ohs[0]);
			}
			return null;
		}

		public void SetStatusBarMessage(string message)
		{
			int n = lblMessage.Text.IndexOf(";");
			if(n != -1)
			{
				lblMessage.Text = lblMessage.Text.Substring(0, n);
			}
			if(lblMessage.Text != "")
			{
				lblMessage.Text += "; ";
			}
			lblMessage.Text += message;
		}

		public void Test()
		{
			Cerebrum.Vocabulary.Desktop.TransformForm dlg1 = new Cerebrum.Vocabulary.Desktop.TransformForm();
			dlg1.DomainContext = this.DomainContext;
			dlg1.m_WMaster = this.m_WMaster;
			dlg1.MdiParent = this.MdiParent;
			dlg1.Show();
		}

		public void ExportCSVFiles()
		{
			Cerebrum.Vocabulary.Desktop.ExportCSVForm dlg = new Cerebrum.Vocabulary.Desktop.ExportCSVForm();
			if(dlg.ShowDialog() == DialogResult.OK)
			{
				WorkInfo.ShowWindow(".   .", this.MdiParent);

				//      .
				string AppPath = System.Windows.Forms.Application.StartupPath;
				Cerebrum.Vocabulary.Desktop.ExportCSV export = new Cerebrum.Vocabulary.Desktop.ExportCSV(this.DomainContext, this.m_WMaster as Cerebrum.Runtime.HFacade);
				if(dlg.IsExportAttributes == true)
				{
					export.ExportZlAttributes(AppPath);
				}
				if(dlg.IsExportSpecifiers == true)
				{
					export.ExportZlSpecifiers(AppPath);
				}
				if(dlg.IsExportTransforms == true)
				{
					export.ExportZlTransforms(AppPath);
				}
				if(dlg.IsExportStatements == true)
				{
					export.ExportZlStatements(AppPath, dlg.IsAddEndings);
				}

				WorkInfo.HideWindow();

				MessageBox.Show("     " + AppPath + @"\Export");

				SetStatusBarMessage("     " + AppPath + @"\Export");
			}
		}

		/*public void ImportZlStatements()
		{
			Cerebrum.Vocabulary.Desktop.FileInfo fi = null;
			Cerebrum.Vocabulary.Desktop.ImportCSV import = new Cerebrum.Vocabulary.Desktop.ImportCSV(this.DomainContext, this.m_WMaster);

			//  ZlStatements.ini
			fi = WorkInFile.LoadFile("ZlStatements.ini", "ZlStatements.ini|ZlStatements.ini|ZlStatements*.ini|ZlStatements*.ini|*.st.ini|*.st.ini|*.*|*.*");
			if(fi != null)
			{
				WorkInfo.ShowWindow(".   ZlStatements.ini.", this.MdiParent);
				import.ImportZlStatements(fi, true);
				WorkInfo.HideWindow();
				SetStatusBarMessage(" ZlStatements.ini .");
			}
			else
			{
				SetStatusBarMessage(" ZlStatements.ini .");
			}
			Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(this.DomainContext, "ZlStatements");
			m_DataGrid1.DataSource = view;
		}*/

		/*public void ImportZlTransforms()
		{
			Cerebrum.Vocabulary.Desktop.FileInfo fi = null;
			Cerebrum.Vocabulary.Desktop.ImportCSV import = new Cerebrum.Vocabulary.Desktop.ImportCSV(this.DomainContext, this.m_WMaster);

			//  ZlTransforms.csv
			fi = WorkInFile.LoadFile("ZlTransforms.csv", "ZlTransforms.csv|ZlTransforms.csv|*.tr.csv|*.tr.csv|*.*|*.*");
			if(fi != null)
			{
				WorkInfo.ShowWindow(".   ZlTransforms.csv.", this.MdiParent);
				fi.FieldsInfo.Add(new Cerebrum.Vocabulary.Desktop.FieldInfo("Name", "Name", "", true, Cerebrum.Vocabulary.Desktop.WayOfDataRecording.Standard));
				fi.FieldsInfo.Add(new Cerebrum.Vocabulary.Desktop.FieldInfo("Wordform", "WordBase", "ZlTransforms", true, Cerebrum.Vocabulary.Desktop.WayOfDataRecording.Indexing));
				fi.FieldsInfo.Add(new Cerebrum.Vocabulary.Desktop.FieldInfo("", "PrimarySpecifier", "ZlTransforms", true, Cerebrum.Vocabulary.Desktop.WayOfDataRecording.Standard));
				fi.FieldsInfo.Add(new Cerebrum.Vocabulary.Desktop.FieldInfo("Specifier", "", "ZlTransforms", false, Cerebrum.Vocabulary.Desktop.WayOfDataRecording.Standard));
				fi.FieldsInfo.Add(new Cerebrum.Vocabulary.Desktop.FieldInfo("Wordform", "", "ZlTransforms", false, Cerebrum.Vocabulary.Desktop.WayOfDataRecording.Standard));
				fi.FieldsInfo.Add(new Cerebrum.Vocabulary.Desktop.FieldInfo("Ending", "", "ZlTransforms", false, Cerebrum.Vocabulary.Desktop.WayOfDataRecording.Standard));
				import.ImportZlTransforms(fi);
				WorkInfo.HideWindow();
				SetStatusBarMessage(" ZlTransforms.csv .");
			}
			else
			{
				SetStatusBarMessage(" ZlTransforms.csv .");
			}
			Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(this.DomainContext, "ZlTransforms");
			m_DataGrid1.DataSource = view;
		}*/

		public void ImportZlSpecifiers()
		{
			Cerebrum.Vocabulary.Desktop.FileInfo fi = null;
			Cerebrum.Vocabulary.Desktop.ImportCSV import = new Cerebrum.Vocabulary.Desktop.ImportCSV(this.DomainContext, this.m_WMaster);

			//  ZlSpecifiers.csv
			fi = WorkInFile.LoadFile("ZlSpecifiers.csv", "ZlSpecifiers.csv|ZlSpecifiers.csv|*.sp.csv|*.sp.csv|*.*|*.*");
			if(fi != null)
			{
				WorkInfo.ShowWindow(".   ZlSpecifiers.csv.", this.MdiParent);
				fi.FieldsInfo.Add(new Cerebrum.Vocabulary.Desktop.FieldInfo("Attributes", "ZlAttributes", "ZlAttributes", true, Cerebrum.Vocabulary.Desktop.WayOfDataRecording.Vector));
				fi.FieldsInfo.Add(new Cerebrum.Vocabulary.Desktop.FieldInfo("Description", "Description", "", true, Cerebrum.Vocabulary.Desktop.WayOfDataRecording.Standard));
				import.ImportZlSpecifiers(fi);
				WorkInfo.HideWindow();
				SetStatusBarMessage(" ZlSpecifiers.csv .");
			}
			else
			{
				SetStatusBarMessage(" ZlSpecifiers.csv .");
			}
			Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(this.DomainContext, "ZlSpecifiers");
			m_DataGrid1.DataSource = view;
		}

		public void ImportZlAttributes()
		{
			Cerebrum.Vocabulary.Desktop.FileInfo fi = null;
			Cerebrum.Vocabulary.Desktop.ImportCSV import = new Cerebrum.Vocabulary.Desktop.ImportCSV(this.DomainContext, this.m_WMaster);

			//  ZlAttributes.csv
			fi = WorkInFile.LoadFile("ZlAttributes.csv", "ZlAttributes.csv|ZlAttributes.csv|*.at.csv|*.at.csv|*.*|*.*");
			if(fi != null)
			{
				WorkInfo.ShowWindow(".   ZlAttributes.csv.", this.MdiParent);
				fi.FieldsInfo.Add(new Cerebrum.Vocabulary.Desktop.FieldInfo("Name", "Name", "", true, Cerebrum.Vocabulary.Desktop.WayOfDataRecording.Standard));
				fi.FieldsInfo.Add(new Cerebrum.Vocabulary.Desktop.FieldInfo("DisplayName", "DisplayName", "", true, Cerebrum.Vocabulary.Desktop.WayOfDataRecording.Standard));
				fi.FieldsInfo.Add(new Cerebrum.Vocabulary.Desktop.FieldInfo("Description", "Description", "", true, Cerebrum.Vocabulary.Desktop.WayOfDataRecording.Standard));
				import.ImportData("ZlAttributes", fi, "");
				WorkInfo.HideWindow();
				SetStatusBarMessage(" ZlAttributes.csv .");
			}
			else
			{
				SetStatusBarMessage(" ZlTAttributes.csv .");
			}
			Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(this.DomainContext, "ZlAttributes");
			m_DataGrid1.DataSource = view;
		}

		private void m_ListBox1_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			string SelTableName = m_ListBox1.SelectedItem.ToString();
			if(m_TableView != null)
			{
				m_TableView.Dispose();
			}
			m_TableView = Cerebrum.Management.Utilites.GetView(this.DomainContext, SelTableName);
			m_DataGrid1.DataSource = m_TableView;
			SetStatusBarMessage(" : " + SelTableName);
		}

		private void m_btnTransform_Click(object sender, System.EventArgs e)
		{
			Cerebrum.Vocabulary.Desktop.TransformForm dlg = new Cerebrum.Vocabulary.Desktop.TransformForm(m_TextBox1.Text);
			dlg.DomainContext = this.DomainContext;
			dlg.m_WMaster = this.m_WMaster;
			dlg.MdiParent = this.MdiParent;
			dlg.Show();
			dlg.Transform();
		}

		private void btnMorphAnalysis_Click(object sender, System.EventArgs e)
		{
			Cerebrum.Vocabulary.Desktop.TransformForm dlg = new Cerebrum.Vocabulary.Desktop.TransformForm(m_TextBox1.Text);
			dlg.DomainContext = this.DomainContext;
			dlg.m_WMaster = this.m_WMaster;
			dlg.MdiParent = this.MdiParent;
			dlg.Show();
			dlg.MorphAnalysis();
		}

		private void btnLemmatize_Click(object sender, System.EventArgs e)
		{
			Cerebrum.Vocabulary.Desktop.TransformForm dlg = new Cerebrum.Vocabulary.Desktop.TransformForm(m_TextBox1.Text);
			dlg.DomainContext = this.DomainContext;
			dlg.m_WMaster = this.m_WMaster;
			dlg.MdiParent = this.MdiParent;
			dlg.Show();
			dlg.Lemmatize();
		}

		/// <summary>
		///            .
		/// </summary>
		/// <param name="vv">.</param>
		/// <param name="AttributeHandle"> .</param>
		private void EngageConnector(Cerebrum.Data.VectorView vv, int AttributeHandle)
		{
			Cerebrum.ObjectHandle h = new Cerebrum.ObjectHandle(AttributeHandle);
			if(vv.ContainsObject(h) == false)
			{
				using(Cerebrum.IComposite comp = this.DomainContext.AttachConnector(h))
				{
					vv.EngageConnector(h, comp);
				}
			}
		}

		public void CreateWordsList()
		{
			int WordCount = 0;
			int StatCount = 0;
			System.Collections.Hashtable htTrans = new Hashtable();

			try
			{
				System.DateTime dtStart = System.DateTime.Now;

				WorkInfo.ShowWindow(".       .", this.MdiParent);

				//      .
				string AppPath = System.Windows.Forms.Application.StartupPath;

				System.IO.FileStream fs2 = new System.IO.FileStream(AppPath + @"\Export\ZlWordsList.log", System.IO.FileMode.Create);
				System.IO.StreamWriter sw2 = new System.IO.StreamWriter(fs2, System.Text.Encoding.GetEncoding("windows-1251"));
				sw2.WriteLine("TransformHandle; TransformName; PrimSpecHandle; PrimarySpecEnding");
				//   .
				using(Cerebrum.Data.TableView tvTrans = Cerebrum.Management.Utilites.GetView(this.DomainContext, "ZlTransforms"))
				{
					//     .
					for(int i1=0; i1<tvTrans.Count; i1++)
					{
						//  .
						Cerebrum.Data.ComponentItemView civTrans = tvTrans[i1];
						using(Cerebrum.IComposite compTrans = this.DomainContext.AttachConnector(civTrans.ObjectHandle))
						{
							//  .
							Cerebrum.Vocabulary.ZlTransform trans = (Cerebrum.Vocabulary.ZlTransform)compTrans.Component;

							ForExportWords fxw = new ForExportWords();
							//     .
							fxw.TransformName = Convert.ToString(Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)compTrans, Cerebrum.Specialized.Concepts.NameAttribute));
							fxw.FlexiaList = ZlTransform.GetFlexiaSortedList((Cerebrum.IContainer)compTrans);
							Cerebrum.ObjectHandle PrimSpecHandle = (Cerebrum.ObjectHandle)trans.GetAttributeVal("PrimarySpecifier");
							fxw.PrimarySpecFlexia = trans.GetAttributeComponent(PrimSpecHandle).ToString();
							htTrans.Add(compTrans.GroundHandle, fxw);

							sw2.WriteLine(civTrans.ObjectHandle.ToString() + "; " + fxw.TransformName + "; " + PrimSpecHandle.ToString() + "; " + fxw.PrimarySpecFlexia);
							sw2.Flush();
						}
					}
				}
				sw2.Flush();
				sw2.Close();
				fs2.Close();

				System.IO.FileStream fs1 = new System.IO.FileStream(AppPath + @"\Export\ZlWordsList.csv", System.IO.FileMode.Create);
				System.IO.StreamWriter sw1 = new System.IO.StreamWriter(fs1, System.Text.Encoding.GetEncoding("windows-1251"));

				//    PrefixModel.
				Cerebrum.ObjectHandle AttrPrefixTransHandle = Cerebrum.Management.Utilites.ResolveHandle(this.DomainContext, "Attributes", "Name", Cerebrum.Vocabulary.Concepts.KnownNames.PrefixModel);
				//    EndingModel.
				Cerebrum.ObjectHandle AttrEndingTransHandle = Cerebrum.Management.Utilites.ResolveHandle(this.DomainContext, "Attributes", "Name", Cerebrum.Vocabulary.Concepts.KnownNames.EndingModel);

				System.Collections.SortedList slTransMapSpecList = new SortedList();
				System.Collections.ArrayList alWords = new ArrayList();
				//   .
				using(Cerebrum.Data.TableView tvStat = Cerebrum.Management.Utilites.GetView(this.DomainContext, "ZlStatements"))
				{
					StatCount = tvStat.Count;

					//     .
					for(int i1=0; i1<tvStat.Count; i1++)
					{
						//  .
						Cerebrum.Data.ComponentItemView civStat = tvStat[i1];
						using(Cerebrum.IComposite compStat = civStat.GetComposite())
						{
							Cerebrum.ObjectHandle PrefixModelGround = ((Cerebrum.Vocabulary.ZlStatement)compStat.Component).PrefixModelGround;
							Cerebrum.ObjectHandle EndingModelGround = ((Cerebrum.Vocabulary.ZlStatement)compStat.Component).EndingModelGround;
							//    .
							ForExportWords fxwP = (ForExportWords)htTrans[PrefixModelGround];
							//    .
							ForExportWords fxwE = (ForExportWords)htTrans[EndingModelGround];
							// 
							string TransCode = "";
							if(fxwP != null)
							{
								TransCode += fxwP.TransformName;
							}
							TransCode += ";";
							if(fxwE != null)
							{
								TransCode += fxwE.TransformName;
							}
							Cerebrum.Vocabulary.ZlStatement stat = (Cerebrum.Vocabulary.ZlStatement)compStat.Component;
							string WordBase = stat.ToString();

							System.Collections.SortedList slSpecList = null;

							//          
							//     ...
							if(slTransMapSpecList.ContainsKey(TransCode) == false)
							{
								// 
								int pi = 0;
								int ei = 0;
								Cerebrum.ObjectHandle SpecHandle = Cerebrum.ObjectHandle.Null;
								slSpecList = new SortedList();
								//       .
								slTransMapSpecList.Add(TransCode, slSpecList);

								for(; ; )
								{
									Cerebrum.ObjectHandle PSH = Cerebrum.ObjectHandle.Null;
									Cerebrum.ObjectHandle ESH = Cerebrum.ObjectHandle.Null;
									string prefix = "";
									string ending = "";
									// 
									if(fxwP != null)
									{
										//      -    ...
										if(fxwP.FlexiaList.Count > pi)
										{
											//    .
											PSH = (Cerebrum.ObjectHandle)fxwP.FlexiaList.GetKey(pi);
											//  .
											prefix = fxwP.FlexiaList.GetByIndex(pi).ToString();
										}
									}
									// 
									if(fxwE != null)
									{
										//      -    ...
										if(fxwE.FlexiaList.Count > ei)
										{
											//    .
											ESH = (Cerebrum.ObjectHandle)fxwE.FlexiaList.GetKey(ei);
											//  .
											ending = fxwE.FlexiaList.GetByIndex(ei).ToString();
										}
									}
									//     ,   .
									Flexias fl = new Flexias();
									//        ...
									if((PSH.ToInt32() == ESH.ToInt32())&&(PSH != Cerebrum.ObjectHandle.Null)&&(ESH != Cerebrum.ObjectHandle.Null))
									{
										//    .
										fl.Prefix = prefix;
										//  .
										fl.Ending = ending;
										//   .
										SpecHandle = PSH;
										//     .
										pi++;
										ei++;
									}
										//        ...
									else if((PSH.ToInt32() < ESH.ToInt32())&&(PSH != Cerebrum.ObjectHandle.Null))
									{
										//     .
										fl.Prefix = prefix;
										//   .
										SpecHandle = PSH;
										//    .
										pi++;
									}
										//        ...
									else if((PSH.ToInt32() > ESH.ToInt32())&&(ESH != Cerebrum.ObjectHandle.Null))
									{
										//     .
										fl.Ending = ending;
										//   .
										SpecHandle = ESH;
										//    .
										ei++;
									}
										//        null...
									else if((PSH == Cerebrum.ObjectHandle.Null)&&(ESH == Cerebrum.ObjectHandle.Null))
									{
										//   .
										break;
									}
										//      null,   
										//     null...
									else if((PSH == Cerebrum.ObjectHandle.Null)&&(ESH != Cerebrum.ObjectHandle.Null))
									{
										//     .
										fl.Ending = ending;
										//   .
										SpecHandle = ESH;
										//    .
										ei++;
									}
										//       null,   
										//    null...
									else if((PSH != Cerebrum.ObjectHandle.Null)&&(ESH == Cerebrum.ObjectHandle.Null))
									{
										//     .
										fl.Prefix = prefix;
										//   .
										SpecHandle = PSH;
										//    .
										pi++;
									}

									//           
									//    .
									slSpecList.Add(SpecHandle, fl);
								}
							}
							else
							{
								//   .
								slSpecList = (System.Collections.SortedList)slTransMapSpecList[TransCode];
							}
							//      .
							for(int i2=0; i2<slSpecList.Count; i2++)
							{
								//           .
								Flexias fl = (Flexias)slSpecList.GetByIndex(i2);
								//   .
								string Word = fl.Prefix + WordBase + fl.Ending;

								alWords.Add(Word);

								WordCount++;
							}
						}
					}
				}

				alWords.Sort();
				for(int i=0; i<alWords.Count; i++)
				{
					string Word = alWords[i].ToString();
					sw1.WriteLine(Word);
					sw1.Flush();
				}

				sw1.Flush();
				sw1.Close();
				fs1.Close();

				WorkInfo.HideWindow();

				System.DateTime dtEnd = System.DateTime.Now;

				MessageBox.Show("   : " + WordCount.ToString() + ";  : " + StatCount.ToString() + "; ,   : " + (dtEnd - dtStart).ToString());

				SetStatusBarMessage("     " + AppPath + @"\Export");
			}
			catch (Exception ex)
			{
				WorkInfo.HideWindow();
				Cerebrum.Windows.Forms.Application.Instance.ProcessException(ex, "");
			}
		}

		private void m_TextBox1_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e)
		{
			if(System.Environment.NewLine.IndexOf(e.KeyChar)>=0)
			{
				e.Handled = true;
				m_btnTransform_Click(sender, e);
			}
		}
	}
}
