/******************************************************************************************************

 Cerebrum.Vocabulary.Library.dll - contains implementation of the application program interface to the 
 morphological database.

 Copyright (C) 2007-2009 Shuklin Dmitry Evgenievich and Boyko Sergey Vasilievich. All rights reserved.

 The Morphology .NET toolkit is free software; you can redistribute it and/or modify it under the terms 
 of the Microsoft Public License (Ms-PL) as published by the Microsoft.

 This toolkit is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
 QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE 
 COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

 See the Microsoft Public License (Ms-PL) for more details.

 You should have received a copy of the Microsoft Public License (Ms-PL) along with this toolkit;
 if not, the Microsoft Public License (Ms-PL) can be found at 
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL

 Please see the LICENSE.txt file for the full license.

 For all inquieries and concerns contact an authors at  cerebrum@shdsoft.ru

 This copyright notice MUST APPEAR in all copies of this file!

 //////////////////////////////////////////////////////////////////////////////////////////////////////

 Cerebrum.Vocabulary.Library.dll -        .
 
 Copyright (C) 2007-2009       .   .

      ;     / 
      Microsoft Public License (Ms-PL).

     ,    ,   - ;
           .  
          .    
 ,        ,   
 .

      Microsoft Public License (Ms-PL)

      Microsoft Public License (Ms-PL)    ; 
   ,  Microsoft Public License (Ms-PL)      
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

       LICENSE.txt

          : cerebrum@shdsoft.ru
 
          !
 
******************************************************************************************************/

using System;

namespace Cerebrum.Vocabulary.Library
{
	public enum KernelObjectKindIdState { Register, Unregister }

	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class Application : Cerebrum.Integrator.Application
	{
		protected static Cerebrum.Vocabulary.Library.Application m_Instance;
		public Cerebrum.Runtime.ISemiotics m_WMaster;
		private string DatabaseFileName;
		private string ActivityFileName;
		private string MergeFileName;
		private static KernelObjectKindIdState KOKIS = KernelObjectKindIdState.Unregister;

		public Application(string databaseFileName, string activityFileName, string mergeFileName)
		{
			DatabaseFileName = databaseFileName;
			ActivityFileName = activityFileName;
			MergeFileName = mergeFileName;
			m_WMaster = null;

			if(KOKIS == KernelObjectKindIdState.Unregister)
			{
				m_Instance = this;
				m_Instance.Initialize();
				m_Instance.PreOpen();
			}
		}

		public Application(string databaseFileName)
		{
			DatabaseFileName = databaseFileName;
			ActivityFileName = "";
			MergeFileName = "";
			m_WMaster = null;

			if(KOKIS == KernelObjectKindIdState.Unregister)
			{
				m_Instance = this;
				m_Instance.Initialize();
				m_Instance.PreOpen();
			}
		}

		public static Cerebrum.Vocabulary.Library.Application Instance
		{
			get
			{
				return m_Instance;
			}
			set
			{
				m_Instance = value;
			}
		}

		public string ApplicationDirectory
		{
			get
			{
				return System.IO.Path.GetDirectoryName(this.GetType().Assembly.Location);
			}
		}
		public override Cerebrum.Integrator.DomainContext CreateMasterContext(string domainName, bool creating)
		{
			Cerebrum.Integrator.DomainContext dc = null;
			dc = new Cerebrum.Integrator.DomainContext(this);
			return dc;
		}

		protected override Cerebrum.Integrator.IContextFactory CreateContextFactory()
		{
			Cerebrum.Integrator.IContextFactory icf = null;

			string primaryDirectory = "";
			string databaseFileName = "";
			if(DatabaseFileName.IndexOf(":") < 0)
			{
				primaryDirectory = this.ApplicationDirectory;
				databaseFileName = System.IO.Path.Combine(primaryDirectory, DatabaseFileName);
			}
			else
			{
				databaseFileName = DatabaseFileName;
			}
			string activityFileName = System.IO.Path.Combine(primaryDirectory, ActivityFileName);
			string templateFileName = System.IO.Path.Combine(primaryDirectory, MergeFileName);

			icf = new Cerebrum.Management.SimpleContextFactory(this, databaseFileName, activityFileName, 4, true, templateFileName, true);
			if(KOKIS == KernelObjectKindIdState.Unregister)
			{
				Cerebrum.Runtime.Semiotics.KernelObjectKindId.Register();
				KOKIS = KernelObjectKindIdState.Register;
			}
			return icf;
		}

		public override void ProcessException(Exception ex, string source)
		{
			System.Console.WriteLine(source);
			System.Console.WriteLine(ex.ToString());
		}

		public void PreOpen()
		{
			string s = string.Empty;
			Cerebrum.ObjectHandle hL = Cerebrum.ObjectHandle.Null;
			Cerebrum.ObjectHandle hH = Cerebrum.ObjectHandle.Null;
			using(Cerebrum.IConnector connector = this.MasterContext.AttachConnector(new Cerebrum.ObjectHandle(60001)))
			{
				if(connector!=null)
				{
					hL = (Cerebrum.ObjectHandle)connector.Component;
				}
			}
			using(Cerebrum.IConnector connector = this.MasterContext.AttachConnector(new Cerebrum.ObjectHandle(60002)))
			{
				if(connector!=null)
				{
					hH = (Cerebrum.ObjectHandle)connector.Component;
				}
			}

			if(hL!=Cerebrum.ObjectHandle.Null)
			{
				this.m_WMaster = Cerebrum.Runtime.LFacade.CreateInstance(this.MasterContext.Workspace);

				this.m_WMaster.ReviveSector(hL);

				s += "ML = " + hL.ToString(); // + System.Environment.NewLine;
			}
			else if(hH!=Cerebrum.ObjectHandle.Null)
			{
				this.m_WMaster = Cerebrum.Runtime.HFacade.CreateInstance(this.MasterContext.Workspace);

				this.m_WMaster.ReviveSector(hH);

				s += "MH = " + hH.ToString(); // + System.Environment.NewLine;
			}
		}

		public override void Shutdown()
		{
			if(this.m_WMaster != null)
			{
				this.m_WMaster.Dispose();
				this.m_WMaster = null;
			}
			base.Shutdown();
		}

		protected override void Dispose(bool disposing)
		{
			base.Dispose (disposing);
		}
	}
}
