/******************************************************************************************************

 Cerebrum.Vocabulary.Library.dll - contains implementation of the application program interface to the 
 morphological database.

 Copyright (C) 2007-2009 Shuklin Dmitry Evgenievich and Boyko Sergey Vasilievich. All rights reserved.

 The Morphology .NET toolkit is free software; you can redistribute it and/or modify it under the terms 
 of the Microsoft Public License (Ms-PL) as published by the Microsoft.

 This toolkit is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
 QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE 
 COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

 See the Microsoft Public License (Ms-PL) for more details.

 You should have received a copy of the Microsoft Public License (Ms-PL) along with this toolkit;
 if not, the Microsoft Public License (Ms-PL) can be found at 
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL

 Please see the LICENSE.txt file for the full license.

 For all inquieries and concerns contact an authors at  cerebrum@shdsoft.ru

 This copyright notice MUST APPEAR in all copies of this file!

 //////////////////////////////////////////////////////////////////////////////////////////////////////

 Cerebrum.Vocabulary.Library.dll -        .
 
 Copyright (C) 2007-2009       .   .

      ;     / 
      Microsoft Public License (Ms-PL).

     ,    ,   - ;
           .  
          .    
 ,        ,   
 .

      Microsoft Public License (Ms-PL)

      Microsoft Public License (Ms-PL)    ; 
   ,  Microsoft Public License (Ms-PL)      
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

       LICENSE.txt

          : cerebrum@shdsoft.ru
 
          !
 
******************************************************************************************************/

using System;

namespace Cerebrum.Vocabulary.Library
{
	public enum TransformMode {Transform, SpellCheck, MorphologicalAnalysis, Lemmatize}

	/// <summary>
	/// Summary description for Transform.
	/// </summary>
	public class Transform
	{
		private Cerebrum.Integrator.DomainContext DC;
		private Cerebrum.Runtime.ISemiotics m_WMaster;

		public Transform(Cerebrum.Integrator.DomainContext dc, Cerebrum.Runtime.ISemiotics WMaster)
		{
			DC = dc;
			m_WMaster = WMaster;
		}

		public Cerebrum.Vocabulary.DataModel.WordInfo[] Lemmatize(string word)
		{
			System.Collections.Hashtable htRes = new System.Collections.Hashtable();

			Cerebrum.Runtime.LFacade lfacade = m_WMaster as Cerebrum.Runtime.LFacade;
			Cerebrum.Runtime.HFacade hfacade = m_WMaster as Cerebrum.Runtime.HFacade;
			if(lfacade!=null)
			{
				TransformLFacade(lfacade, word, htRes, TransformMode.Lemmatize);
			}
			else if(hfacade!=null)
			{
				TransformHFacade(hfacade, word, htRes, TransformMode.Lemmatize);
			}
			else
			{
				TransformOld(word, htRes, TransformMode.Lemmatize);
			}
			Cerebrum.Vocabulary.DataModel.WordInfo[] res = new Cerebrum.Vocabulary.DataModel.WordInfo[htRes.Count];
			int index = 0;
			foreach(System.Collections.DictionaryEntry de in htRes)
			{
				res[index] = (Cerebrum.Vocabulary.DataModel.WordInfo)de.Value;
				index++;
			}
			return res;
		}

		public Cerebrum.Vocabulary.DataModel.WordInfo[] TransformWord(string word)
		{
			System.Collections.Hashtable htRes = new System.Collections.Hashtable();

			Cerebrum.Runtime.LFacade lfacade = m_WMaster as Cerebrum.Runtime.LFacade;
			Cerebrum.Runtime.HFacade hfacade = m_WMaster as Cerebrum.Runtime.HFacade;
			if(lfacade!=null)
			{
				TransformLFacade(lfacade, word, htRes, TransformMode.Transform);
			}
			else if(hfacade!=null)
			{
				TransformHFacade(hfacade, word, htRes, TransformMode.Transform);
			}
			else
			{
				TransformOld(word, htRes, TransformMode.Transform);
			}
			Cerebrum.Vocabulary.DataModel.WordInfo[] res = new Cerebrum.Vocabulary.DataModel.WordInfo[htRes.Count];
			int index = 0;
			foreach(System.Collections.DictionaryEntry de in htRes)
			{
				res[index] = (Cerebrum.Vocabulary.DataModel.WordInfo)de.Value;
				index++;
			}
			return res;
		}

		public bool WordSpellCheck(string word)
		{
			bool res = false;
			System.Collections.Hashtable htRes1 = new System.Collections.Hashtable();

			Cerebrum.Runtime.LFacade lfacade = m_WMaster as Cerebrum.Runtime.LFacade;
			Cerebrum.Runtime.HFacade hfacade = m_WMaster as Cerebrum.Runtime.HFacade;
			if(lfacade!=null)
			{
				res = TransformLFacade(lfacade, word, htRes1, TransformMode.MorphologicalAnalysis);
			}
			else if(hfacade!=null)
			{
				res = TransformHFacade(hfacade, word, htRes1, TransformMode.MorphologicalAnalysis);
			}
			else
			{
				res = TransformOld(word, htRes1, TransformMode.MorphologicalAnalysis);
			}
			if(res == true)
			{
				if(htRes1.Count <=0)
				{
					res = false;
				}
			}
			return res;
		}

		public Cerebrum.Vocabulary.DataModel.WordInfo[] MorphologicalAnalysisOfWord(string word)
		{
			bool res = false;
			System.Collections.Hashtable htRes1 = new System.Collections.Hashtable();
			System.Collections.ArrayList alRes2 = new System.Collections.ArrayList();

			Cerebrum.Runtime.LFacade lfacade = m_WMaster as Cerebrum.Runtime.LFacade;
			Cerebrum.Runtime.HFacade hfacade = m_WMaster as Cerebrum.Runtime.HFacade;
			if(lfacade!=null)
			{
				res = TransformLFacade(lfacade, word, htRes1, TransformMode.MorphologicalAnalysis);
			}
			else if(hfacade!=null)
			{
				res = TransformHFacade(hfacade, word, htRes1, TransformMode.MorphologicalAnalysis);
			}
			else
			{
				res = TransformOld(word, htRes1, TransformMode.MorphologicalAnalysis);
			}
			//       ...
			if(res == true)
			{
				//      .
				foreach(System.Collections.DictionaryEntry de in htRes1)
				{
					//       .
					System.Collections.ArrayList alWordFormInfoList = new System.Collections.ArrayList();
					//    .
					Cerebrum.Vocabulary.DataModel.WordInfo wi = (Cerebrum.Vocabulary.DataModel.WordInfo)de.Value;
					//      .
					for(int i=0; i<wi.WordFormInfoList.Count; i++)
					{
						//   .
						Cerebrum.Vocabulary.DataModel.WordFormInfo wfi = (Cerebrum.Vocabulary.DataModel.WordFormInfo)wi.WordFormInfoList[i];
						//         ...
						if(wfi.Word == word)
						{
							//         .
							alWordFormInfoList.Add(wfi);
						}
					}
					//        ...
					if(alWordFormInfoList.Count > 0)
					{
						//      .
						wi.WordFormInfoList = alWordFormInfoList;
						//      .
						alRes2.Add(wi);
					}
				}
			}
			return (Cerebrum.Vocabulary.DataModel.WordInfo[])alRes2.ToArray(typeof(Cerebrum.Vocabulary.DataModel.WordInfo));
		}

		private bool TransformLFacade(Cerebrum.Runtime.LFacade wmaster, string wordform, System.Collections.Hashtable htRes, TransformMode mode)
		{
			bool res = false;

			Cerebrum.ObjectHandle endingModelAttrHandle = Cerebrum.Management.Utilites.ResolveHandle(DC, "Attributes", "Name", Cerebrum.Vocabulary.Concepts.KnownNames.EndingModel);

			Cerebrum.ObjectHandle [][]waves = new ObjectHandle[wordform.Length][];
			wmaster.StrokeRoot();
			for(int i = 0; i < wordform.Length; i++)
			{
				if(!wmaster.StrokeNode(wordform[i], true))
				{
					break;
				}
				Cerebrum.ObjectHandle [] wave = wmaster.SelectWave();
				if(wave==null || wave.Length < 1)
				{
					break;
				}
				waves[i] = wave;
			}
			for(int i = 0; i < wordform.Length; i++)
			{
				Cerebrum.ObjectHandle [] wave = waves[i];
				if(wave==null)
				{
					break;
				}

				Cerebrum.ObjectHandle [] ohEs = null;
				string ending = string.Empty;
				string wordbase = wordform;
				if(i + 1 < wordform.Length)
				{
					wordbase = wordform.Substring(0, i + 1);
					ending = wordform.Substring(i + 1, wordform.Length - i - 1);
				}

				for(int j=0; j<wave.Length; j++)
				{
					Cerebrum.Collections.Specialized.HandleCollection cll = null;
					using(Cerebrum.IComposite compW = wmaster.AttachConnector(wave[j]))
					{
						if(compW is System.IO.Stream)
						{
							System.IO.Stream stm = compW as System.IO.Stream;
							if(stm.Length > 0)
							{
								cll = new Cerebrum.Collections.Specialized.HandleCollection();
								cll.Serialize(Cerebrum.SerializeDirection.Load, null, stm);
							}
						}
					}
					if(cll!=null)
					{
						if(ohEs==null)
						{
							ohEs = wmaster.ParseToken(ending, true);
						}
						if(ohEs==null || ohEs.Length < 1)
						{
							break;
						}
						else
						{
							res = true;
							if((mode == TransformMode.Transform)||(mode == TransformMode.MorphologicalAnalysis)||(mode == TransformMode.Lemmatize))
							{
								foreach(Cerebrum.ObjectHandle ohS in cll)
								{
									// ZlStatement
									using(Cerebrum.IComposite compS = DC.AttachConnector(ohS))
									{
										// ZlTransform
										using(Cerebrum.IComposite compT = ((Cerebrum.IContainer)compS).AttachConnector(endingModelAttrHandle))
										{
											foreach(Cerebrum.ObjectHandle ohE in ohEs)
											{
												Cerebrum.ObjectHandle [] keys = (compT as Cerebrum.Runtime.NativeWarden).KeysOf(ohE);
												if(keys.Length > 0)
												{
													DoTransform(ohS, null, compT, wordbase, ending, mode, htRes);
												}
											}
										}
									}
								}
							}
							else if(mode == TransformMode.SpellCheck)
							{
								return res;
							}
						}
					}
				}
			}
			return res;
		}

		private bool TransformHFacade(Cerebrum.Runtime.HFacade wmaster, string wordform, System.Collections.Hashtable htRes, TransformMode mode)
		{
			bool res = false;

			Cerebrum.Vocabulary.ZlStatement.DomainProperties statementProperties = Cerebrum.Vocabulary.ZlStatement.GetProperties(DC);

			System.Collections.Hashtable hshLines = new System.Collections.Hashtable();
			Cerebrum.ObjectHandle[] line = new ObjectHandle[wordform.Length + 1];
			for(int i = 0; i <= wordform.Length; i++)
			{
				if(i==0)
				{
					line[0] = wmaster.StrokeRoot();
				}
				else
				{
					line[i] = wmaster.StrokeNode(wordform[i-1], line[i-1], true);
				}
				if(line[i]==Cerebrum.ObjectHandle.Null)
				{
					break;
				}

				string ending = string.Empty;
				if(i < wordform.Length)
				{
					ending = wordform.Substring(i, wordform.Length - i);
				}

				foreach(Cerebrum.ObjectHandle root in wmaster.SelectLine(line[i]))
				{
					Cerebrum.ObjectHandle [] wave = wmaster.SelectWave(line[i], root);
					if(wave==null || wave.Length < 1)
					{
						continue;
					}

					Cerebrum.ObjectHandle [] ohEs = null;

					for(int j=0; j<wave.Length; j++)
					{
						string wordbase = null;
						Cerebrum.Collections.Specialized.HandleCollection cll = null;

						using(Cerebrum.IComposite compW = wmaster.AttachConnector(wave[j]))
						{
							if(compW is Cerebrum.Runtime.NativeWarden)
							{
								Cerebrum.Runtime.NativeWarden nwW = (Cerebrum.Runtime.NativeWarden)compW;
								//  ZlStatements  ZlTansforms   
								foreach(System.Collections.DictionaryEntry deW in nwW)
								{
									Cerebrum.ObjectHandle ohS = (Cerebrum.ObjectHandle)deW.Key;

									if(cll == null)
									{
										cll = new Cerebrum.Collections.Specialized.HandleCollection();
									}
									cll.Add(ohS);
								}
							}
							if(compW is System.IO.Stream)
							{
								System.IO.Stream stm = compW as System.IO.Stream;
								if(stm.Length > 0)
								{
									if(cll == null)
									{
										cll = new Cerebrum.Collections.Specialized.HandleCollection();
									}
									cll.Serialize(Cerebrum.SerializeDirection.Load, null, stm);
								}
							}
						}

						if(cll!=null)
						{
							res = true;
							if((mode == TransformMode.Transform)||(mode == TransformMode.MorphologicalAnalysis)||(mode == TransformMode.Lemmatize))
							{
								foreach(Cerebrum.ObjectHandle ohS in cll)
								{
									using(Cerebrum.IComposite comp0 = DC.AttachConnector(ohS))
									{
										// ZlStatement
										if(comp0.Component is Cerebrum.Vocabulary.ZlStatement)
										{
											if(ohEs==null)
											{
												ohEs = ParseHToken(wmaster, ending, true);
												if(ohEs==null)
												{
													ohEs = new Cerebrum.ObjectHandle[0];
												}
											}
											if(ohEs!=null && ohEs.Length > 0)
											{
												System.Collections.Hashtable hsh = null;
												System.Collections.Hashtable lst = null;
												Cerebrum.ObjectHandle PrefixModelGround = (comp0.Component as Cerebrum.Vocabulary.ZlStatement).PrefixModelGround;
												if(root!=line[0])
												{
													hsh = (System.Collections.Hashtable)hshLines[root];
													if(hsh!=null)
													{
														lst = (System.Collections.Hashtable)hsh[PrefixModelGround];
													}
												}
												// ZlTransform
												Cerebrum.IComposite compE = ((Cerebrum.IContainer)comp0).AttachConnector(statementProperties.EndingModelHandle);
												try
												{
													foreach(Cerebrum.ObjectHandle ohE in ohEs)
													{
														bool valid = false;
														Cerebrum.ObjectHandle [] keys = null;
														keys = (compE as Cerebrum.Runtime.NativeWarden).KeysOf(ohE);
														if(lst!=null && keys!=null)
														{
															foreach(Cerebrum.ObjectHandle key in keys)
															{
																if(lst.ContainsKey(key))
																{
																	valid = true;
																}
															}
														}
														else
														{
															if(root==line[0] /* && PrefixModelGround==Cerebrum.ObjectHandle.Null */)
															{
																//TODO here can be check for empty prefix
																if(keys!=null && keys.Length > 0)
																{
																	valid = true;
																}
																else
																{
																	if(compE==null)
																	{
																		valid = true;
																	}
																}
															}
														}
														if(valid)
														{
															if(wordbase==null)
															{
																wordbase = Cerebrum.Vocabulary.Desktop.Utilites.FilterNodeText(wmaster.ObtainPath(line[i], root, wave[j]));
															}
															Cerebrum.IComposite compP = null;
															if(PrefixModelGround!=Cerebrum.ObjectHandle.Null)
															{
																compP = wmaster.AttachConnector(PrefixModelGround);
															}
															try
															{
																DoTransform(ohS, compP, compE, wordbase, ending, mode, htRes);
															}
															finally
															{
																if(compP!=null)
																{
																	compP.Dispose();
																	compP = null;
																}
															}
														}
													}
												}
												finally
												{
													if(compE!=null)
													{
														((System.IDisposable)compE).Dispose();
														compE = null;
													}
												}
											}
										}
										else if(comp0.Component is Cerebrum.Vocabulary.ZlTransform)
										{
											if(root==line[0])
											{
												Cerebrum.ObjectHandle [] keys = (comp0 as Cerebrum.Runtime.NativeWarden).KeysOf(wave[j]);

												if(keys.Length > 0)
												{
													System.Collections.Hashtable hsh = (System.Collections.Hashtable)hshLines[line[i]];
													if(hsh==null)
													{
														hsh = new System.Collections.Hashtable();
														hshLines[line[i]] = hsh;
													}

													System.Collections.Hashtable lst = (System.Collections.Hashtable)hsh[comp0.GroundHandle];
													if(lst==null)
													{
														lst = new System.Collections.Hashtable();
														hsh[comp0.GroundHandle] = lst;
													}
													foreach(Cerebrum.ObjectHandle key in keys)
													{
														if(!lst.ContainsKey(key))
														{
															lst.Add(key, string.Empty);
														}
													}
												}
											}
										}
									}
								}
							}
							else if(mode == TransformMode.SpellCheck)
							{
								return res;
							}
						}
					}
				}
			}
			return res;
		}

		private bool TransformOld(string wordform, System.Collections.Hashtable htRes, TransformMode mode)
		{
			bool res = false;
			
			Cerebrum.ObjectHandle endingModelAttrHandle = Cerebrum.Management.Utilites.ResolveHandle(DC, "Attributes", "Name", Cerebrum.Vocabulary.Concepts.KnownNames.EndingModel);

			//  
			using(Cerebrum.IComposite compW = Cerebrum.Vocabulary.Desktop.Utilites.AttachStringQuoted(m_WMaster, wordform, true))
			{
				if(compW==null)
				{
					return res;
				}

				if(compW is Cerebrum.Runtime.NativeWarden)
				{
					Cerebrum.Runtime.NativeWarden nwW = (Cerebrum.Runtime.NativeWarden)compW;

					if(nwW.Count > 0)
					{
						res = true;
						if((mode == TransformMode.Transform)||(mode == TransformMode.MorphologicalAnalysis)||(mode == TransformMode.Lemmatize))
						{
							//  ZlStatements   
							foreach(System.Collections.DictionaryEntry deW in nwW)
							{
								Cerebrum.ObjectHandle ohS = (Cerebrum.ObjectHandle)deW.Key;

								// ZlStatement
								using(Cerebrum.IComposite compS = nwW.AttachConnector(ohS))
								{
									// ZlTransform
									using(Cerebrum.IComposite compT = ((Cerebrum.IContainer)compS).AttachConnector(endingModelAttrHandle))
									{
										DoTransform(ohS, null, compT, wordform, "", mode, htRes);
									}
								}
							}
						}
						else if(mode == TransformMode.SpellCheck)
						{
							return res;
						}
					}
				}
				else if(compW is System.IO.Stream)
				{
					System.IO.Stream stm = compW as System.IO.Stream;
					Cerebrum.Collections.Specialized.HandleCollection cll = new Cerebrum.Collections.Specialized.HandleCollection();
					cll.Serialize(Cerebrum.SerializeDirection.Load, null, stm);

					if(cll.Count > 0)
					{
						res = true;
						if((mode == TransformMode.Transform)||(mode == TransformMode.MorphologicalAnalysis)||(mode == TransformMode.Lemmatize))
						{
							foreach(Cerebrum.ObjectHandle ohS in cll)
							{
								// ZlStatement
								using(Cerebrum.IComposite compS = DC.AttachConnector(ohS))
								{
									// ZlTransform
									using(Cerebrum.IComposite compT = ((Cerebrum.IContainer)compS).AttachConnector(endingModelAttrHandle))
									{
										DoTransform(ohS, null, compT, wordform, "", mode, htRes);
									}
								}
							}
						}
						else if(mode == TransformMode.SpellCheck)
						{
							return res;
						}
					}
				}
			}
			return res;
		}

		private void DoTransform(Cerebrum.ObjectHandle StatHandle, Cerebrum.IComposite compP, Cerebrum.IComposite compE, string wordbase, string Ending, TransformMode mode, System.Collections.Hashtable htRes)
		{
			string PrefixTransName = "";
			if(compP != null)
			{
				PrefixTransName = Convert.ToString(compP.Component);
			}
			
			string EndingTransName = "";
			if(compE != null)
			{
				EndingTransName = Convert.ToString(compE.Component);
			}

			System.Collections.IDictionaryEnumerator enumerator = null;
			bool b2 = false, bP = false, bE = false;

			//    ...
			if(mode == TransformMode.Lemmatize)
			{
				object PrimSpecObj = null;
				Cerebrum.ObjectHandle PrimSpecHandleP = Cerebrum.ObjectHandle.Null;
				Cerebrum.ObjectHandle PrimSpecHandleE = Cerebrum.ObjectHandle.Null;
				System.Collections.Hashtable ht = new System.Collections.Hashtable();

				//      ...
				if(compE != null)
				{
					//   .
					Cerebrum.Vocabulary.ZlTransform transE = (Cerebrum.Vocabulary.ZlTransform)compE.Component;
					//     .
					PrimSpecObj = transE.GetAttributeVal("PrimarySpecifier");
					if(PrimSpecObj != null)
					{
						PrimSpecHandleE = (Cerebrum.ObjectHandle)PrimSpecObj;
						//    .
						bE = true;
					}
				}

				//      ...
				if(compP!=null)
				{
					//   .
					Cerebrum.Vocabulary.ZlTransform transP = (Cerebrum.Vocabulary.ZlTransform)compP.Component;
					//     .
					PrimSpecObj = transP.GetAttributeVal("PrimarySpecifier");
					if(PrimSpecObj != null)
					{
						PrimSpecHandleP = (Cerebrum.ObjectHandle)PrimSpecObj;
						//       
						//    ...
						if(PrimSpecHandleP == PrimSpecHandleE)
						{
							//    .
							bP = true;
						}
						else
						{
							//       null.
							PrimSpecHandleP = Cerebrum.ObjectHandle.Null;
						}
					}
				}

				//       ...
				if((bP == true)&&(bE == true))
				{
					//   2- .
					b2 = true;
				}

				//   .
				Cerebrum.ObjectHandle[] val = new ObjectHandle[2];
				val[0] = PrimSpecHandleP;
				val[1] = PrimSpecHandleE;
				ht.Add(PrimSpecHandleE, val);

				//  IDictionaryEnumerator  .
				enumerator = (System.Collections.IDictionaryEnumerator)ht.GetEnumerator();
			}
			else
			{
				if(compP!=null && compE!=null)
				{
					enumerator = Cerebrum.Runtime.Enumerators.CreateOuterJoinEnumerator(new Cerebrum.IContainer[] {(Cerebrum.IContainer)compP, (Cerebrum.IContainer)compE});
					b2 = true;
				}
				else if(compE!=null)
				{
					enumerator = (System.Collections.IDictionaryEnumerator)((System.Collections.IEnumerable)compE).GetEnumerator();
					bE = true;
				}
				else if(compP!=null)
				{
					enumerator = (System.Collections.IDictionaryEnumerator)((System.Collections.IEnumerable)compP).GetEnumerator();
					bP = true;
				}
			}

			Cerebrum.ObjectHandle ohP = Cerebrum.ObjectHandle.Null, ohE = Cerebrum.ObjectHandle.Null;

			//  ZlSpecifiers   
			enumerator.Reset();
			while(enumerator.MoveNext())
			{
				Cerebrum.ObjectHandle ohV = (Cerebrum.ObjectHandle)enumerator.Key;

				if(b2)
				{
					Cerebrum.ObjectHandle[] ohS = (Cerebrum.ObjectHandle[])enumerator.Value;
					ohP = Cerebrum.ObjectHandle.Null.Equals(ohS[0])?Cerebrum.ObjectHandle.Null:ohV;
					ohE = Cerebrum.ObjectHandle.Null.Equals(ohS[1])?Cerebrum.ObjectHandle.Null:ohV;
				}
				else if(bP)
				{
					ohP = ohV;
				}
				else if(bE)
				{
					ohE = ohV;
				}

				string specName = null;
				Cerebrum.Vocabulary.ZlSpecifier spec = null;
				System.Collections.ArrayList al1 = new System.Collections.ArrayList();
				using(Cerebrum.IComposite compSp = DC.AttachConnector(ohV))
				{
					if(compSp.Component is Cerebrum.Vocabulary.ZlSpecifier)
					{
						specName = Convert.ToString(Cerebrum.Integrator.Utilites.ObtainComponent((Cerebrum.IContainer)compSp, Cerebrum.Specialized.Concepts.NameAttribute));

						spec = (Cerebrum.Vocabulary.ZlSpecifier)compSp.Component;
						Cerebrum.ObjectHandle[] AttrHandles = spec.GetChildrenNodes("ZlAttributes");
						//     .
						for(int i=0; i<AttrHandles.Length; i++)
						{
							//  ObjectHandle  .
							Cerebrum.ObjectHandle h = AttrHandles[i];
							using (Cerebrum.IComposite comp1 = DC.AttachConnector(h))
							{
								Cerebrum.Vocabulary.ZlAttribute attr = (Cerebrum.Vocabulary.ZlAttribute)comp1.Component;

								Cerebrum.Vocabulary.DataModel.AttributeInfo ai = new Cerebrum.Vocabulary.DataModel.AttributeInfo();
								ai.Name = attr.GetAttributeVal("Name").ToString();
								ai.DisplayName = attr.GetAttributeVal("DisplayName").ToString();
								ai.Description = attr.GetAttributeVal("Description").ToString();
								
								al1.Add(ai);
							}
						}
					}
				}
				if(specName!=null && specName != string.Empty)
				{
					string prefix = string.Empty;
					string ending = string.Empty;
					//   prefix
					if(ohP==ohV)
						using(Cerebrum.IComposite compT = ((Cerebrum.IContainer)compP).AttachConnector(ohP))
						{
							if(m_WMaster is Cerebrum.Runtime.HFacade)
							{
								prefix = Cerebrum.Vocabulary.Desktop.Utilites.FilterNodeText((m_WMaster as Cerebrum.Runtime.HFacade).ObtainText(compT));
							}
							else
							{
								prefix = Convert.ToString(compT.Component);
							}
						}
					//   ending
					if(ohE==ohV)
						using(Cerebrum.IComposite compT = ((Cerebrum.IContainer)compE).AttachConnector(ohE))
						{
							if(m_WMaster is Cerebrum.Runtime.HFacade)
							{
								ending = Cerebrum.Vocabulary.Desktop.Utilites.FilterNodeText((m_WMaster as Cerebrum.Runtime.HFacade).ObtainText(compT));
							}
							else
							{
								ending = Convert.ToString(compT.Component);
							}
						}

					bool IsAdded = true;
					if((mode == TransformMode.MorphologicalAnalysis)&&(Ending != ending))
					{
						IsAdded = false;
					}
					if(IsAdded == true)
					{
						//       .
						Cerebrum.Vocabulary.DataModel.WordFormInfo wfi = new Cerebrum.Vocabulary.DataModel.WordFormInfo();
						//     .
						wfi.AttributesList = al1;
						//  .
						wfi.Statement = StatHandle.ToString();
						//    .
						wfi.Word = prefix + wordbase + ending;
						//        .
						wfi.SpecifierName = specName;

						//        .
						Cerebrum.Vocabulary.DataModel.WordInfo wi = null;
						//    .
						string Key = PrefixTransName + ";" + EndingTransName;
						//       ...
						if(htRes.ContainsKey(Key) == false)
						{
							//       .
							wi = new Cerebrum.Vocabulary.DataModel.WordInfo();
							//    .
							wi.PrefixTransform = PrefixTransName;
							//    .
							wi.EndingTransform = EndingTransName;
							//      .
							htRes.Add(Key, wi);
						}
						else
						{
							//       .
							wi = (Cerebrum.Vocabulary.DataModel.WordInfo)htRes[Key];
						}
						//       .
						wi.WordFormInfoList.Add(wfi);
					}
				}
			}
			if(enumerator is System.IDisposable)
			{
				((System.IDisposable)enumerator).Dispose();
			}
		}

		private Cerebrum.ObjectHandle [] ParseHToken(Cerebrum.Runtime.HFacade wmaster, string text, bool readOnly)
		{
			Cerebrum.ObjectHandle root = wmaster.StrokeRoot();
			Cerebrum.ObjectHandle curr = root;
			for(int i = 0; i < text.Length; i++)
			{
				curr = wmaster.StrokeNode(text[i], curr, readOnly);
				if(curr==Cerebrum.ObjectHandle.Null)
				{
					break;
				}
			}
			Cerebrum.ObjectHandle [] wave = wmaster.SelectWave(curr, root);
			return wave;
		}
	}
}
