/******************************************************************************************************

 Cerebrum.Vocabulary.dll - contains implementation of the morphology database objects .

 Copyright (C) 2007-2009 Shuklin Dmitry Evgenievich and Boyko Sergey Vasilievich. All rights reserved.

 The Morphology .NET toolkit is free software; you can redistribute it and/or modify it under the terms 
 of the Microsoft Public License (Ms-PL) as published by the Microsoft.

 This toolkit is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
 QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE 
 COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

 See the Microsoft Public License (Ms-PL) for more details.

 You should have received a copy of the Microsoft Public License (Ms-PL) along with this toolkit;
 if not, the Microsoft Public License (Ms-PL) can be found at 
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL

 Please see the LICENSE.txt file for the full license.

 For all inquieries and concerns contact an authors at  cerebrum@shdsoft.ru

 This copyright notice MUST APPEAR in all copies of this file!

 //////////////////////////////////////////////////////////////////////////////////////////////////////

 Cerebrum.Vocabulary.dll -      .
 
 Copyright (C) 2007-2009       .   .

      ;     / 
      Microsoft Public License (Ms-PL).

     ,    ,   - ;
           .  
          .    
 ,        ,   
 .

      Microsoft Public License (Ms-PL)

      Microsoft Public License (Ms-PL)    ; 
   ,  Microsoft Public License (Ms-PL)      
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

       LICENSE.txt

          : cerebrum@shdsoft.ru
 
          !
 
******************************************************************************************************/

using System;

namespace Cerebrum.Vocabulary
{
	/// <summary>
	/// Summary description for ZlSpecifiers.
	/// </summary>
	public class ZlSpecifier : Cerebrum.Reflection.AttributeDescriptor
	{
		public ZlSpecifier()
		{
		}

		protected override void SetConnector(Cerebrum.SerializeDirection direction, Cerebrum.IConnector connector)
		{
			base.SetConnector(direction, connector);
			switch (direction)
			{
				case Cerebrum.SerializeDirection.Init:
				{
					//    ZlAttributes
					CreateVector("ZlAttributes");
					break;
				}
			}
		}

		public object GetAttributeVal(string name)
		{
			Cerebrum.ObjectHandle h;
			object obj = null;

			if (this.DomainContext != null)
			{
				h = Cerebrum.Management.Utilites.ResolveHandle(this.DomainContext, "Attributes", "Name", name);
				obj = this.GetAttributeComponent(h);
			}
			return obj;
		}

		public bool SetAttributeVal(string name, object val)
		{
			Cerebrum.ObjectHandle h;
			bool res = false;

			if (this.DomainContext != null)
			{
				h = Cerebrum.Management.Utilites.ResolveHandle(this.DomainContext, "Attributes", "Name", name);
				this.SetAttributeComponent(h, val);
				res = true;
			}
			return res;
		}

		/// <summary>
		///  .
		/// </summary>
		/// <param name="VectorHandle"> .</param>
		public void CreateVector(Cerebrum.ObjectHandle VectorHandle)
		{
			//  NativeWarden.
			using (Cerebrum.Runtime.NativeWarden warden = (this.GetChildComponents() as Cerebrum.Runtime.NativeWarden))
			{
				//    .
				warden.Newobj(VectorHandle, Cerebrum.Runtime.KernelObjectKindId.Warden);
				using (Cerebrum.IComposite comp = warden.AttachConnector(VectorHandle))
				{
					if (comp is Cerebrum.Runtime.ICompositeEx)
					{
						((Cerebrum.Runtime.ICompositeEx)comp).FastenGround();
					}
				}
				warden.FastenGround();
			}
		}

		/// <summary>
		///  .
		/// </summary>
		/// <param name="VectorName"> </param>
		public void CreateVector(string VectorName)
		{
			//   -.
			Cerebrum.ObjectHandle VectorHandle = Cerebrum.Management.Utilites.ResolveHandle(this.DomainContext, "Attributes", "Name", VectorName);
			//  .
			CreateVector(VectorHandle);
		}

		/// <summary>
		///     .
		/// </summary>
		/// <param name="vectorName"> .</param>
		/// <returns> .</returns>
		public Cerebrum.ObjectHandle[] GetChildrenNodes(string vectorName)
		{
			Cerebrum.ObjectHandle h;
			Cerebrum.Runtime.NativeWarden nw = null;
			Cerebrum.ObjectHandle[] ObjectHandleArray = null;

			using (Cerebrum.Runtime.NativeWarden warden = (this.GetChildComponents() as Cerebrum.Runtime.NativeWarden))
			{
				h = Cerebrum.Management.Utilites.ResolveHandle(this.DomainContext, "Attributes", "Name", vectorName);
				using(Cerebrum.IComposite myobj = warden.AttachConnector(h))
				{
					if (myobj != null)
					{
						nw = (Cerebrum.Runtime.NativeWarden)myobj;
						ObjectHandleArray = new ObjectHandle[nw.Count];
						int i = 0;
						foreach(System.Collections.DictionaryEntry de in nw)
						{
							ObjectHandleArray[i] = (Cerebrum.ObjectHandle)de.Key;
							i++;
						}
					}
				}
			}
			return ObjectHandleArray;
		}

		/// <summary>
		///   ObjectHandle  .
		/// </summary>
		/// <param name="h"> .</param>
		public void AddChildrenNode(string vectorName, Cerebrum.ObjectHandle h)
		{
			Cerebrum.ObjectHandle hV = Cerebrum.ObjectHandle.Null;
			Cerebrum.Runtime.NativeWarden nw = null;

			using (Cerebrum.Runtime.NativeWarden warden = (this.GetChildComponents() as Cerebrum.Runtime.NativeWarden))
			{
				hV = Cerebrum.Management.Utilites.ResolveHandle(this.DomainContext, "Attributes", "Name", vectorName);
				using(Cerebrum.IComposite myobj = warden.AttachConnector(hV))
				{
					if (myobj != null)
					{
						nw = (Cerebrum.Runtime.NativeWarden)myobj;
						using (Cerebrum.Runtime.NativeSector sector = this.DomainContext.GetSector())
						{
							using (Cerebrum.IComposite comp = sector.AttachConnector(h))
							{
								nw.EngageConnector(h, comp);
							}
						}
					}
				}
			}
		}

		/// <summary>
		///  ObjectHandle  .
		/// </summary>
		/// <param name="h"> .</param>
		public void DelChildrenNode(string vectorName, Cerebrum.ObjectHandle h)
		{
			Cerebrum.ObjectHandle hV = Cerebrum.ObjectHandle.Null;
			Cerebrum.Runtime.NativeWarden nw = null;

			using (Cerebrum.Runtime.NativeWarden warden = (this.GetChildComponents() as Cerebrum.Runtime.NativeWarden))
			{
				hV = Cerebrum.Management.Utilites.ResolveHandle(this.DomainContext, "Attributes", "Name", vectorName);
				using(Cerebrum.IComposite myobj = warden.AttachConnector(hV))
				{
					if (myobj != null)
					{
						nw = (Cerebrum.Runtime.NativeWarden)myobj;
						nw.RemoveConnector(h);
					}
				}
			}
		}

		/// <summary>
		///      ZlAttributes.
		/// </summary>
		public Cerebrum.ObjectHandle[] ZlAttributes
		{
			get
			{
				return GetChildrenNodes("ZlAttributes");
			}
		}
	}
}
