/******************************************************************************************************

 Cerebrum.Vocabulary.dll - contains implementation of the morphology database objects .

 Copyright (C) 2007-2009 Shuklin Dmitry Evgenievich and Boyko Sergey Vasilievich. All rights reserved.

 The Morphology .NET toolkit is free software; you can redistribute it and/or modify it under the terms 
 of the Microsoft Public License (Ms-PL) as published by the Microsoft.

 This toolkit is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
 QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE 
 COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

 See the Microsoft Public License (Ms-PL) for more details.

 You should have received a copy of the Microsoft Public License (Ms-PL) along with this toolkit;
 if not, the Microsoft Public License (Ms-PL) can be found at 
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL

 Please see the LICENSE.txt file for the full license.

 For all inquieries and concerns contact an authors at  cerebrum@shdsoft.ru

 This copyright notice MUST APPEAR in all copies of this file!

 //////////////////////////////////////////////////////////////////////////////////////////////////////

 Cerebrum.Vocabulary.dll -      .
 
 Copyright (C) 2007-2009       .   .

      ;     / 
      Microsoft Public License (Ms-PL).

     ,    ,   - ;
           .  
          .    
 ,        ,   
 .

      Microsoft Public License (Ms-PL)

      Microsoft Public License (Ms-PL)    ; 
   ,  Microsoft Public License (Ms-PL)      
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

       LICENSE.txt

          : cerebrum@shdsoft.ru
 
          !
 
******************************************************************************************************/

using System;

namespace Cerebrum.Vocabulary
{
	/// <summary>
	/// Summary description for ZlStatements.
	/// </summary>
	public class ZlStatement : Cerebrum.Vocabulary.BaseNeuron, Cerebrum.IPersistent, Cerebrum.IDiscovery
	{
		private static object proertiesKey = new object();

		public class DomainProperties
		{
			public Cerebrum.ObjectHandle WordBaseHandle;
			public Cerebrum.ObjectHandle PrefixModelHandle;
			public Cerebrum.ObjectHandle EndingModelHandle;
		}

		protected override void SetConnector(SerializeDirection direction, IConnector connector)
		{
			m_IsChanged = false;
			m_Properties = null;
			m_ToStringValue = null;
			PrefixModelGround = Cerebrum.ObjectHandle.Null;
			EndingModelGround = Cerebrum.ObjectHandle.Null;
			WordBaseGround = Cerebrum.ObjectHandle.Null;

			base.SetConnector (direction, connector);

			using(Cerebrum.IContainerEvents events = ((Cerebrum.Runtime.IConnectorEx)connector).GetContainer() as Cerebrum.IContainerEvents)
			{
				events.ConnectorAttach += new ConnectorAttachEventHandler(OnAttachConnector);
				events.ConnectorEngage += new ConnectorEngageEventHandler(OnEngageConnector);
			}
		}

		private string m_ToStringValue;

		public static ZlStatement.DomainProperties GetProperties(Cerebrum.Integrator.DomainContext dc)
		{
			ZlStatement.DomainProperties _properties = dc.Properties[ZlStatement.proertiesKey] as ZlStatement.DomainProperties;
			if(_properties==null)
			{
				_properties = new DomainProperties();
				_properties.WordBaseHandle = Cerebrum.Management.Utilites.ResolveHandle(dc, "Attributes", "Name", "WordBase");
				_properties.PrefixModelHandle = Cerebrum.Management.Utilites.ResolveHandle(dc, "Attributes", "Name", Cerebrum.Vocabulary.Concepts.KnownNames.PrefixModel);
				_properties.EndingModelHandle = Cerebrum.Management.Utilites.ResolveHandle(dc, "Attributes", "Name", Cerebrum.Vocabulary.Concepts.KnownNames.EndingModel);
				dc.Properties[ZlStatement.proertiesKey] = _properties;
			}
			return _properties;
		}

		private ZlStatement.DomainProperties m_Properties;
		private ZlStatement.DomainProperties Properties
		{
			get
			{
				if(this.m_Properties==null)
				{
					m_Properties = GetProperties(this.DomainContext);
				}
				return m_Properties;
			}
		}

		public override string ToString()
		{
			if(m_ToStringValue==null)
			{
				m_ToStringValue = Convert.ToString(this.GetAttributeComponent(this.Properties.WordBaseHandle));
				if(m_ToStringValue==null)
				{
					m_ToStringValue = string.Empty;
				}
			}
			return m_ToStringValue;
			//return base.ToString ();
		}

		private bool m_IsChanged;
		private Cerebrum.ObjectHandle WordBaseGround;
		public Cerebrum.ObjectHandle PrefixModelGround;
		public Cerebrum.ObjectHandle EndingModelGround;

		#region IPersistent Members

		public void Serialize(Cerebrum.SerializeDirection direction, IConnector connector, System.IO.Stream stream)
		{
			switch(direction)
			{
				case Cerebrum.SerializeDirection.Load:
					this.PrefixModelGround = Cerebrum.ObjectHandle.ReadFrom(stream);
					this.EndingModelGround = Cerebrum.ObjectHandle.ReadFrom(stream);
					this.WordBaseGround = Cerebrum.ObjectHandle.ReadFrom(stream);
					m_IsChanged = false;
					break;
				case Cerebrum.SerializeDirection.Save:
					Cerebrum.ObjectHandle.WriteTo(stream, this.PrefixModelGround);
					Cerebrum.ObjectHandle.WriteTo(stream, this.EndingModelGround);
					Cerebrum.ObjectHandle.WriteTo(stream, this.WordBaseGround);
					m_IsChanged = false;
					break;
			}
		}

		#endregion

		#region IDiscovery Members

		public bool IsChanged
		{
			get
			{
				return m_IsChanged;
			}
		}

		#endregion

		private static void OnEngageConnector(object sender, ConnectorEngageEventArgs e)
		{
			if(!e.Handled)
			{
				Cerebrum.ObjectHandle ground = Cerebrum.ObjectHandle.Null;
				ZlStatement statement = e.Container.Component as ZlStatement;
				bool PrefixModelMode = false;
				bool EndingModelMode = false;
				bool WordBaseMode = false;
				if( (PrefixModelMode = statement.Properties.PrefixModelHandle==e.PlasmaHandle) || (EndingModelMode = statement.Properties.EndingModelHandle==e.PlasmaHandle) || (WordBaseMode = statement.Properties.WordBaseHandle==e.PlasmaHandle))
				{
					ground = ((Cerebrum.IComposite)e.Dependent).GroundHandle;
					if(Cerebrum.ObjectHandle.Null==ground)
					{
						((Cerebrum.Runtime.ICompositeEx)e.Dependent).FastenGround();
						ground = ((Cerebrum.IComposite)e.Dependent).GroundHandle;
					}
					if(PrefixModelMode)
					{
						statement.PrefixModelGround = ground;
						statement.m_IsChanged = true;
						e.Handled = true;
					}
					else if(EndingModelMode)
					{
						statement.EndingModelGround = ground;
						statement.m_IsChanged = true;
						e.Handled = true;
					}
					else if(WordBaseMode)
					{
						statement.WordBaseGround = ground;
						statement.m_IsChanged = true;
						e.Handled = true;
					}
				}
				/*if(Cerebrum.ObjectHandle.Null==ground || e.Handled == false)
				{
					throw new System.Exception("Internal error");
				}*/
			}
		}

		private static void OnAttachConnector(object sender, Cerebrum.ConnectorAttachEventArgs e)
		{
			if(!e.Handled)
			{
				ZlStatement statement = e.Container.Component as ZlStatement;
				if(statement.Properties.PrefixModelHandle==e.PlasmaHandle)
				{
					if(Cerebrum.ObjectHandle.Null==statement.PrefixModelGround)
					{
						e.Dependent = null;
					}
					else
					{
						e.Dependent = (Cerebrum.IComposite)((Cerebrum.Runtime.NativeDomain)e.Container.Workspace).AttachComposite(statement.PrefixModelGround, e.PlasmaHandle, Cerebrum.ObjectHandle.Null);
					}
					e.Handled = true;
				}
				else if(statement.Properties.EndingModelHandle==e.PlasmaHandle)
				{
					if(Cerebrum.ObjectHandle.Null==statement.EndingModelGround)
					{
						e.Dependent = null;
					}
					else
					{
						e.Dependent = (Cerebrum.IComposite)((Cerebrum.Runtime.NativeDomain)e.Container.Workspace).AttachComposite(statement.EndingModelGround, e.PlasmaHandle, Cerebrum.ObjectHandle.Null);
					}
					e.Handled = true;
				}
				else if(statement.Properties.WordBaseHandle==e.PlasmaHandle)
				{
					if(Cerebrum.ObjectHandle.Null==statement.WordBaseGround)
					{
						e.Dependent = null;
					}
					else
					{
						e.Dependent = (Cerebrum.IComposite)((Cerebrum.Runtime.NativeDomain)e.Container.Workspace).AttachComposite(statement.WordBaseGround, e.PlasmaHandle, Cerebrum.ObjectHandle.Null);
					}
					e.Handled = true;
				}
			}
		}

	}
}
