/******************************************************************************************************

 Cerebrum.Vocabulary.dll - contains implementation of the morphology database objects .

 Copyright (C) 2007-2009 Shuklin Dmitry Evgenievich and Boyko Sergey Vasilievich. All rights reserved.

 The Morphology .NET toolkit is free software; you can redistribute it and/or modify it under the terms 
 of the Microsoft Public License (Ms-PL) as published by the Microsoft.

 This toolkit is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
 QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE 
 COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

 See the Microsoft Public License (Ms-PL) for more details.

 You should have received a copy of the Microsoft Public License (Ms-PL) along with this toolkit;
 if not, the Microsoft Public License (Ms-PL) can be found at 
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL

 Please see the LICENSE.txt file for the full license.

 For all inquieries and concerns contact an authors at  cerebrum@shdsoft.ru

 This copyright notice MUST APPEAR in all copies of this file!

 //////////////////////////////////////////////////////////////////////////////////////////////////////

 Cerebrum.Vocabulary.dll -      .
 
 Copyright (C) 2007-2009       .   .

      ;     / 
      Microsoft Public License (Ms-PL).

     ,    ,   - ;
           .  
          .    
 ,        ,   
 .

      Microsoft Public License (Ms-PL)

      Microsoft Public License (Ms-PL)    ; 
   ,  Microsoft Public License (Ms-PL)      
 http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

       LICENSE.txt

          : cerebrum@shdsoft.ru
 
          !
 
******************************************************************************************************/

using System;

namespace Cerebrum.Vocabulary
{
	/// <summary>
	/// Summary description for ZlTransforms.
	/// </summary>
	public class ZlTransform : Cerebrum.Vocabulary.BaseNeuron
	{
		protected override void SetConnector(SerializeDirection direction, IConnector connector)
		{
			m_ToStringValue = null;
			base.SetConnector (direction, connector);
		}

		private string m_ToStringValue;

		public override string ToString()
		{
			if(m_ToStringValue==null)
			{
				m_ToStringValue = Convert.ToString(this.GetAttributeComponent(Cerebrum.Specialized.Concepts.NameAttribute));
				if(m_ToStringValue==null)
				{
					m_ToStringValue = string.Empty;
				}
			}
			return m_ToStringValue;
			//return base.ToString ();
		}

		/// <summary>
		///    ,    .
		/// </summary>
		/// <returns></returns>
		public Cerebrum.ObjectHandle[] GetSpecifierHandlesList()
		{
			Cerebrum.Integrator.DomainContext dc = this.DomainContext;
			System.Collections.ArrayList al = new System.Collections.ArrayList();
			using(Cerebrum.IContainer cont = this.GetChildComponents())
			{
				foreach(System.Collections.DictionaryEntry de in ((System.Collections.IEnumerable)cont))
				{
					Cerebrum.ObjectHandle AttrHandle = (Cerebrum.ObjectHandle)de.Key;
					using(Cerebrum.IComposite comp = dc.AttachConnector(AttrHandle))
					{
						if(comp.Component.GetType() == typeof(Cerebrum.Vocabulary.ZlSpecifier))
						{
							al.Add(AttrHandle);
						}
					}
				}
			}
			al.Sort();
			return (Cerebrum.ObjectHandle[])al.ToArray(typeof(Cerebrum.ObjectHandle));
		}	

		/// <summary>
		///       .
		/// </summary>
		/// <returns></returns>
		public System.Collections.Specialized.StringCollection GetEndingsList()
		{
			System.Collections.Specialized.StringCollection scEndingsList = new System.Collections.Specialized.StringCollection();
			//    ,    .
			Cerebrum.ObjectHandle[] SpecHandlesList = GetSpecifierHandlesList();
			//      .
			for(int i2=0; i2<SpecHandlesList.Length; i2++)
			{
				Cerebrum.ObjectHandle SpecHandle = SpecHandlesList[i2];
				//   .
				object val = GetAttributeComponent(SpecHandle);
				string Ending = val.ToString();
				if(scEndingsList.Contains(Ending) == false)
				{
					scEndingsList.Add(Ending);
				}
				Ending = null;
				val = null;
			}
			return scEndingsList;
		}

		/// <summary>
		///        .
		/// </summary>
		/// <returns></returns>
		public static string[] GetEndingsSortedList(Cerebrum.IContainer container)
		{
			System.Collections.ArrayList alEndingsList = new System.Collections.ArrayList();
			System.Collections.IEnumerator enumerator = ((System.Collections.IEnumerable)container).GetEnumerator();
			try
			{
				enumerator.Reset();
				while(enumerator.MoveNext())
				{
					System.Collections.DictionaryEntry de = (System.Collections.DictionaryEntry)enumerator.Current;
					Cerebrum.ObjectHandle AttrHandle = (Cerebrum.ObjectHandle)de.Key;
					using(Cerebrum.IComposite comp = ((Cerebrum.IConnector)container).Workspace.AttachConnector(AttrHandle))
					{
						if(comp.Component is Cerebrum.Vocabulary.ZlSpecifier)
						{
							//   .
							alEndingsList.Add(string.Intern(Convert.ToString(Cerebrum.Integrator.Utilites.ObtainComponent(container, AttrHandle))));
						}
					}
				}
			}
			finally
			{
				if(enumerator is System.IDisposable)
				{
					((System.IDisposable)enumerator).Dispose();
				}
				enumerator = null;
			}
			alEndingsList.Sort();
			return (string[])alEndingsList.ToArray(typeof(string));
		}

		/// <summary>
		///       .
		/// </summary>
		/// <returns></returns>
		public static System.Collections.SortedList GetFlexiaSortedList(Cerebrum.IContainer container)
		{
			System.Collections.SortedList slFlexiaList = new System.Collections.SortedList();
			System.Collections.IEnumerator enumerator = ((System.Collections.IEnumerable)container).GetEnumerator();

			try
			{
				enumerator.Reset();
				while(enumerator.MoveNext())
				{
					System.Collections.DictionaryEntry de = (System.Collections.DictionaryEntry)enumerator.Current;
					Cerebrum.ObjectHandle AttrHandle = (Cerebrum.ObjectHandle)de.Key;
					using(Cerebrum.IComposite comp = ((Cerebrum.IConnector)container).Workspace.AttachConnector(AttrHandle))
					{
						if(comp.Component is Cerebrum.Vocabulary.ZlSpecifier)
						{
							//  .
							string Flexia = string.Intern(Convert.ToString(Cerebrum.Integrator.Utilites.ObtainComponent(container, AttrHandle)));
							// 
							slFlexiaList.Add(AttrHandle, Flexia);
						}
					}
				}
			}
			finally
			{
				if(enumerator is System.IDisposable)
				{
					((System.IDisposable)enumerator).Dispose();
				}
				enumerator = null;
			}
			return slFlexiaList;
		}
	}
}
